module cube_op

use cubefile
use electrostatic

implicit none

interface assignment(=)
 module procedure cub_eq_cub
end interface

contains

subroutine geo2cube(g,c,nx,ny,nz)
type(geo), intent(in)       :: g
type(cube), intent(out)     :: c
integer                     :: i
integer, intent(in)         :: nx,ny,nz

if(g%fmt.ne."xyz") then
 print*,"only xyz implemented up to now"
 stop
endif


allocate(c%rho(nx,ny,nz))
allocate(c%pos(g%n,3))
allocate(c%dumr(g%n))
allocate(c%dumn(g%n))

do i=1,g%n
c%dumr(i)=1.d0*atnum(g%name(i))
c%dumn(i)=atnum(g%name(i))
c%pos(i,1:3)=g%p(i,1:3)/0.529177d0
enddo
c%natoms=g%n
c%nx=nx
c%ny=ny
c%nz=nz
c%dx(1,1:3)=g%b(1,1:3)/c%nx/0.529177d0
c%dx(2,1:3)=g%b(2,1:3)/c%ny/0.529177d0
c%dx(3,1:3)=g%b(3,1:3)/c%nz/0.529177d0
c%offset=0.d0
call set_cube(c)

end subroutine geo2cube

subroutine set_cube(s)
type(cube), intent(inout)      :: s
s%dv = dabs( s%dx(3,1)*(s%dx(1,2)*s%dx(2,3) - s%dx(1,3)*s%dx(2,2)) +  &
             s%dx(3,2)*(s%dx(1,1)*s%dx(2,3) - s%dx(1,3)*s%dx(2,1)) +  &
             s%dx(3,3)*(s%dx(1,1)*s%dx(2,2) - s%dx(1,2)*s%dx(2,1))   )

s%q=sum(s%rho(:,:,:)) * s%dv

end subroutine set_cube

subroutine cube2geo(c,g)

type(cube), intent(in)      :: c
type(geo), intent(out)      :: g
integer                     :: i
character(len=2)            :: n(100)

n="H "
n(2)="He"
n(3)="Li"
n(4)="Be"
n(5)="Li"
n(6)="C "
n(7)="N "
n(8)="O "
n(13)="Al"
n(29)="Cu"
n(34)="Se"
n(53)="I "
n(73)="Ta"
n(79)="Au"
allocate(g%p(c%natoms,3))
allocate(g%name(c%natoms))
g%n=c%natoms
g%b(1,1:3)=c%dx(1,1:3)*c%nx
g%b(2,1:3)=c%dx(2,1:3)*c%ny
g%b(3,1:3)=c%dx(3,1:3)*c%nz
g%b=g%b*0.529177d0
do i=1,c%natoms
g%p(i,1:3)=c%pos(i,1:3)*0.529177d0
g%name(i)=n(c%dumn(i))
enddo

end subroutine cube2geo

real*8 function avg_cube(c,p,iav)
type(cube), intent(in)        :: c
integer, intent(in)           :: p(3),iav
integer                       :: ii,jj,kk,p0(3)

avg_cube=0.d0

do ii=-iav,iav
 do jj=-iav,iav
   do kk=-iav,iav
      p0(1)=p(1)+ii
      p0(2)=p(2)+jj
      p0(3)=p(3)+kk
      call pbci(p0,c%nx,c%ny,c%nz)
      avg_cube=avg_cube+c%rho(p0(1),p0(2),p0(3))
   enddo
 enddo
enddo
avg_cube=avg_cube/(2*iav+1)**3

end function avg_cube

subroutine collapse_cube(c,n,n1,n2)

type(cube), intent(inout)  :: c
integer                    :: i,j,k,is,n,n1,n2
real*8, allocatable        :: rho(:,:,:)

allocate(rho(c%nx,c%ny,c%nz))

rho=0.d0

do i=1,c%nx
 do j=1,c%ny
  do k=1,c%nz

      do is=n1,n2
        rho(i,j,k)=rho(i,j,k)+c%rho(i,j,(k-1)*n+is)**2
      enddo

  enddo
 enddo
enddo


deallocate(c%rho)
allocate(c%rho(c%nx,c%ny,c%nz))
c%rho=rho
deallocate(rho)

c%rho = c%rho*2.d0
end subroutine collapse_cube

subroutine traslacube(c,t)
type(cube), intent(inout)  :: c
real*8, intent(in)         :: t(3)
real*8                     :: tf(3)
integer                    :: ti(3)
integer                    :: i,j,k,pi(3)
real*8, allocatable        :: rho(:,:,:)

call ijk(ti,t,c%dx)
tf(1)=ti(1)*c%dx(1,1)+ti(2)*c%dx(2,1)+ti(3)*c%dx(3,1)
tf(2)=ti(1)*c%dx(1,2)+ti(2)*c%dx(2,2)+ti(3)*c%dx(3,2)
tf(3)=ti(1)*c%dx(1,3)+ti(2)*c%dx(2,3)+ti(3)*c%dx(3,3)

do i=1,c%natoms
  c%pos(i,1:3)=c%pos(i,1:3)+tf
enddo

allocate(rho(c%nx,c%ny,c%nz))

do i=1,c%nx
 do j=1,c%ny
  do k=1,c%nz
   pi(1)=i+ti(1) 
   pi(2)=j+ti(2) 
   pi(3)=k+ti(3) 
   call pbci(pi,c%nx,c%ny,c%nz)
   rho(pi(1),pi(2),pi(3))=c%rho(i,j,k)
  enddo
 enddo
enddo

c%rho=rho
deallocate(rho)


end subroutine traslacube

subroutine taglia_cube_i(c,imin,imax,jmin,jmax,kmin,kmax)

type(cube), intent(inout) :: c
integer, intent(in)        :: imin,imax,jmin,jmax,kmin,kmax
real*8                    ::  v(3)
integer                    :: i,j,k,pi(3),ii,jj,kk
real*8, allocatable        :: rho(:,:,:)




c%nx=imax-imin+1
c%ny=jmax-jmin+1
c%nz=kmax-kmin+1
allocate(rho(c%nx,c%ny,c%nz))
rho(1:c%nx,1:c%ny,1:c%nz)=c%rho(imin:imax,jmin:jmax,kmin:kmax)
v=c%dx(1,1:3)*(imin-1) + c%dx(2,1:3)*(jmin-1) + c%dx(3,1:3)*(kmin-1)
deallocate(c%rho)
allocate(c%rho(c%nx,c%ny,c%nz))
c%rho = rho
deallocate(rho)

do i=1,c%natoms
 c%pos(i,1:3)=c%pos(i,1:3)-v
enddo

end subroutine taglia_cube_i

subroutine taglia_cube(c,t)

type(cube), intent(inout) :: c
real*8, intent(in)        :: t
real*8                    :: s,v(3)
integer                    :: i,j,k,pi(3),ii,jj,kk
integer                    :: imin,imax,jmin,jmax,kmin,kmax
real*8, allocatable        :: rho(:,:,:)

imin=c%nx
imax=0
jmin=c%ny
jmax=0
kmin=c%nz
kmax=0

do i=1,c%nx

!s=sum(abs(c%rho(i,1:c%ny,1:c%nz)))

 do j=1,c%ny
  do k=1,c%nz

    s=abs(c%rho(i,j,k))

    if(s.ge.t)then
     if(i.lt.imin)imin=i
     if(i.gt.imax)imax=i
    endif

  enddo
 enddo

enddo

do j=1,c%ny

!s=sum(abs(c%rho(imin:imax,j,1:c%nz)))
 do i=imin,imax
  do k=1,c%nz

    s=abs(c%rho(i,j,k))

    if(s.ge.t)then
     if(j.lt.jmin)jmin=j
     if(j.gt.jmax)jmax=j
    endif

  enddo
 enddo

enddo

do k=1,c%nz

! s=sum(abs(c%rho(imin:imax,jmin:jmax,k)))

 do i=imin,imax
  do j=jmin,jmax

    s=abs(c%rho(i,j,k))

    if(s.ge.t)then
     if(k.lt.kmin)kmin=k
     if(k.gt.kmax)kmax=k
    endif
 
  enddo
 enddo

enddo

c%nx=imax-imin+1
c%ny=jmax-jmin+1
c%nz=kmax-kmin+1
allocate(rho(c%nx,c%ny,c%nz))
rho(1:c%nx,1:c%ny,1:c%nz)=c%rho(imin:imax,jmin:jmax,kmin:kmax)
v=c%dx(1,1:3)*(imin-1) + c%dx(2,1:3)*(jmin-1) + c%dx(3,1:3)*(kmin-1)
deallocate(c%rho)
allocate(c%rho(c%nx,c%ny,c%nz))
c%rho = rho
deallocate(rho)

do i=1,c%natoms
 c%pos(i,1:3)=c%pos(i,1:3)-v
enddo


end subroutine taglia_cube

subroutine reduce_cube(c,n)

type(cube), intent(inout) :: c
integer, intent(in)        :: n
integer                    :: i,j,k,pi(3),ii,jj,kk
real*8, allocatable        :: rho(:,:,:)

allocate(rho(c%nx/n,c%ny/n,c%nz/n))
do i=1,c%nx/n
 do j=1,c%ny/n
  do k=1,c%nz/n

   do ii=1,n
    do jj=1,n
     do kk=1,n
        pi(1)=(n*(i-1)+1)+ii
        pi(2)=(n*(j-1)+1)+jj
        pi(3)=(n*(k-1)+1)+kk
        call pbci(pi,c%nx,c%ny,c%nz)
        rho(i,j,k)=rho(i,j,k)+c%rho(pi(1),pi(2),pi(3))
      enddo
     enddo
    enddo
    rho(i,j,k)=rho(i,j,k)/(n*n*n)

  enddo
 enddo
enddo

deallocate(c%rho)
allocate(c%rho(c%nx/n,c%ny/n,c%nz/n))
c%nx=c%nx/n
c%ny=c%ny/n
c%nz=c%nz/n
c%dx=c%dx*n
c%rho=rho
deallocate(rho)

end subroutine reduce_cube

subroutine cub_eq_cub(a,b)

type(cube), intent(in)    :: b
type(cube), intent(inout) :: a

if(associated(a%pos)) then
 call free_cube(a)
endif


allocate(a%pos(b%natoms,3))
allocate(a%dumr(b%natoms))
allocate(a%dumn(b%natoms))
allocate(a%rho(b%nx,b%ny,b%nz))

a%pos=b%pos
a%dumr=b%dumr
a%dumn=b%dumn
a%rho=b%rho
a%natoms=b%natoms
a%offset=b%offset
a%nx=b%nx
a%ny=b%ny
a%nz=b%nz
a%dv=b%dv
a%q=b%q
a%dx=b%dx


end subroutine cub_eq_cub

subroutine free_cube(s)
type(cube), intent(inout)   ::s

deallocate(s%rho)
deallocate(s%pos)
deallocate(s%dumn)
deallocate(s%dumr)

end subroutine free_cube

subroutine mpole_cube(c,l,d,pole)
type(cube), intent(in)    :: c
integer, intent(in)       :: l
complex*16, intent(out)   :: d(-l:l)
real*8, intent(in)        :: pole(3)
real*8, allocatable       :: rho(:,:)
integer                   :: i,j,k,vi(3),ii
integer, allocatable      :: index(:,:)
real*8                    :: v(3),basis(3,3),dx,dy,dz,minrho

minrho=0.000001d0
allocate(index(c%nx*c%ny*c%nz,3))
basis(1,1:3)=c%dx(1,1:3)*c%nx
basis(2,1:3)=c%dx(2,1:3)*c%ny
basis(3,1:3)=c%dx(3,1:3)*c%nz

ii=0
do i=1,c%nx
 do j=1,c%ny
  do k=1,c%nz     !taglia il cube a minrho
   if(c%rho(i,j,k).gt.minrho) then
     ii=ii+1
     index(ii,1)=i
     index(ii,2)=j
     index(ii,3)=k
   endif
  enddo
 enddo
enddo

allocate(rho(ii,4))

do i=1,ii
   vi=index(i,1:3)
   call xyz(vi,v,c%dx)  
   call vettore(pole,v,dx,dy,dz,basis)
   rho(i,1)=dx
   rho(i,2)=dy
   rho(i,3)=dz
   rho(i,4)=-c%rho(vi(1),vi(2),vi(3))*c%dv
enddo

call multipole(rho,l,d)
deallocate(rho)
deallocate(index)

end subroutine mpole_cube

subroutine mpole_cube_at(c,l,d,pole)
type(cube), intent(in)    :: c
integer, intent(in)       :: l
complex*16, intent(out)   :: d(-l:l)
real*8, intent(in)        :: pole(3)
real*8, allocatable       :: rho(:,:)
integer                   :: i,j,k,vi(3),ii
integer, allocatable      :: index(:,:)
real*8                    :: v(3),basis(3,3),dx,dy,dz,minrho

minrho=1.0d-10
allocate(index(c%nx*c%ny*c%nz,3))
basis(1,1:3)=c%dx(1,1:3)*c%nx
basis(2,1:3)=c%dx(2,1:3)*c%ny
basis(3,1:3)=c%dx(3,1:3)*c%nz

ii=0
do i=1,c%nx
 do j=1,c%ny
  do k=1,c%nz     !taglia il cube a minrho
   if(c%rho(i,j,k).gt.minrho) then
     ii=ii+1
     index(ii,1)=i
     index(ii,2)=j
     index(ii,3)=k
   endif
  enddo
 enddo
enddo

allocate(rho(ii+c%natoms,4))

do i=1,ii
   vi=index(i,1:3)
   call xyz(vi,v,c%dx)  
   call vettore(pole,v,dx,dy,dz,basis)
   rho(i,1)=dx
   rho(i,2)=dy
   rho(i,3)=dz
   rho(i,4)=-c%rho(vi(1),vi(2),vi(3))*c%dv
enddo

do i=1,c%natoms
   v=c%pos(i,1:3)
   call vettore(pole,v,dx,dy,dz,basis)
   rho(ii+i,1)=dx
   rho(ii+i,2)=dy
   rho(ii+i,3)=dz
   rho(ii+i,4)=c%dumn(i)
enddo

call multipole(rho,l,d)
deallocate(rho)
deallocate(index)

end subroutine mpole_cube_at

subroutine rmpole_cube_at(c,l,d,pole)
type(cube), intent(in)    :: c
integer, intent(in)       :: l
real*8, intent(out)   :: d(-l:l)
real*8, intent(in)        :: pole(3)
real*8, allocatable       :: rho(:,:)
integer                   :: i,j,k,vi(3),ii
integer, allocatable      :: index(:,:)
real*8                    :: v(3),basis(3,3),dx,dy,dz,minrho

minrho=1.0d-10
allocate(index(c%nx*c%ny*c%nz,3))
basis(1,1:3)=c%dx(1,1:3)*c%nx
basis(2,1:3)=c%dx(2,1:3)*c%ny
basis(3,1:3)=c%dx(3,1:3)*c%nz

ii=0
do i=1,c%nx
 do j=1,c%ny
  do k=1,c%nz     !taglia il cube a minrho
   if(c%rho(i,j,k).gt.minrho) then
     ii=ii+1
     index(ii,1)=i
     index(ii,2)=j
     index(ii,3)=k
   endif
  enddo
 enddo
enddo

allocate(rho(ii+c%natoms,4))

do i=1,ii
   vi=index(i,1:3)
   call xyz(vi,v,c%dx)  
   call vettore(pole,v,dx,dy,dz,basis)
   rho(i,1)=dx
   rho(i,2)=dy
   rho(i,3)=dz
   rho(i,4)=-c%rho(vi(1),vi(2),vi(3))*c%dv
enddo

do i=1,c%natoms
   v=c%pos(i,1:3)
   call vettore(pole,v,dx,dy,dz,basis)
   rho(ii+i,1)=dx
   rho(ii+i,2)=dy
   rho(ii+i,3)=dz
   rho(ii+i,4)=c%dumn(i)
enddo

call rmultipole(rho,l,d)
deallocate(rho)
deallocate(index)

end subroutine rmpole_cube_at

end module cube_op
