program control_file
  use eigenstate_utils
  use operations
 ! use geo
 
  implicit none

  ! Input related variables
  character(len=10000000) :: my_fmt
  character(len=100) :: buffer, label, outfile, vectorout,verbos ,cubeofile,date, time, mode,strucv,posfile,ofilename
  character(len=100000) :: puffer
  integer :: apos,pos, jval, ndim,mdim,i,j,k,jj,kk,nkt,np,nb ,iat,itemp,imode
  integer, parameter :: fh = 15
  integer, parameter :: fg = 6
  integer, parameter :: ff = 7
  integer, parameter :: dm = 500 ! maximum lines that can be read
  integer     :: ios =0,ioss=0,line=0,lwork,info,freymin,freymax,nat,nmode,nstat,ncf,nx,ny,nz,ntemp
  integer     :: nkpoint=1 , jkpoint=1,reason,NstationFiles,iStation
!  integer, dimension(3) :: today, now

  ! Control file variables
  real :: pill,  maxen , meann , vpert=0.0d0,r
  real*8  , dimension(dm,dm) :: rarrn, rbrrn
  double precision :: mass, cp,hh ,hg, a, b,eref,ci,si,ymin ,ymax, w ,kpoint
  integer, dimension(dm) :: vector, ivec
  double precision , dimension(2,3) :: daphon
  double precision , dimension(3) :: shift 
  double precision , dimension(dm,3) :: ipd
  double precision , dimension(dm) :: rvec,rvecs,dvec, dbasa, frq, kpointvec,temparr, harc=0.d0
  double precision , dimension(dm,6) :: splinc=0.d0
  double precision , dimension(2,2) :: darr2, dbrr2
  double precision , dimension(3,3) :: darr3, dbrr3,uc,ucc
  double precision , dimension(dm,dm) :: bas, darrn, ddrrn,ekpoints
  double precision , dimension(:), allocatable :: ndimvec,work,amode,bmode,erefmode,freea
  double precision , dimension(:,:), allocatable :: ndimarr,epot,ekin,hndimarr,hhndimarr,posxyz,possxyz,mmode,sigmode,emode
  double precision , dimension(:,:,:), allocatable :: pmode,cfmode,tdimarr,hpmode
  double precision , dimension(:,:,:,:), allocatable :: dxarr,xarr


  character(LEN=100), dimension(:), allocatable :: stationFileNames,atomLabels
  character(LEN=500), dimension(:), allocatable :: hcfmode

  open(fh, file='enaliz.dat')

  ndim=100 !default value of number of basis functions
  posfile='snap.xyz'
  vectorout='eigenvecs.dat'
  verbos='low'
  mode='harmonic'
  cubeofile='vibellips'
  shift=(/ 0.d0, 0.d0, 0.d0 /)
  nx=0 ;   ny=0 ;   nz=0
  temparr=-10000.d0 ; temparr(1)=273.d0
  call init_periodic_table()

  ! ios is negative if an end of record condition is encountered or if
  ! an endfile condition was detected.  It is positive if an error was
  ! detected.  ios is zero otherwise.

  if (ios /= 0) then
    print *, 'Input naliz.dat does not exist'
    stop
  end if
  
  jval=1 ; ipd=0.0000 ; ymax=0.d0
  do while (ios == 0)
     read(fh, '(A)', iostat=ios) puffer
     if (ios == 0) then
        line = line + 1
        ! Find the first instance of whitespace.  Split label and data.
        apos = scan(puffer,"abcdefghijklmnopqrstuvwxyz-123456789")
        pos = scan(puffer(apos:), ' ')
        label = trim(puffer(apos:pos+apos-1))
        buffer = trim(puffer(apos+pos:))
   !       write(*,*) scan(puffer,"abcdefghijklmnopqrstuvwxyz-123456789") ,label
   !    write(*,*) trim(label), ' ',trim(buffer)
        select case (label)
        case ('mode')
           read(buffer, *, iostat=ios) mode ! harmonic, spline
           print *, 'Read mode: ', mode
        case ('verbos')
           read(buffer, *, iostat=ios) verbos ! mid high low
           print *, 'Read verbos: ', verbos
        case ('shift')
           read(buffer, *, iostat=ios) shift
           print *, 'Read shift: ', shift
        case ('npix')
           read(buffer, *, iostat=ios) nx,ny,nz
           print *, 'Read npix: ', nx,ny,nz
        case ('ndim')
           read(buffer, *, iostat=ios) ndim
           print *, 'Read ndim: ', ndim
        case ('temp')
           read(buffer, *, iostat=ios) temparr
           ntemp=count(temparr.gt.-10000.d0)
           print *, 'Read temp: ', temparr(1:ntemp)
        case ('posfile')
           read(buffer, *, iostat=ios) posfile
           print *, 'Read posfile: ', posfile
        case ('outfile')
           read(buffer, *, iostat=ios) outfile
           print *, 'Read outfile: ', outfile
        case default
           read(puffer, *, iostat=ioss) ipd(jval,:) 
           if  (scan(label,'abcdefghijklmnopqrstuvwxyz#').eq.0) then ! test if object is a number
            write(*,"(1x,'ipd(',I4,'): ', F18.10,1x,I5,1x,F18.10)") jval,ipd(jval,1),int(ipd(jval,2)),ipd(jval,3)
            jval=jval+1
           else
            print *, 'Skipping invalid label at line', line
           end if
        end select
     end if
  end do
jval=jval-1
close(fh)
 

ntemp=count(temparr.gt.-10000.d0)


allocate (hhndimarr(ndim,ndim),hndimarr(ndim,ndim),ndimarr(ndim,ndim),epot(ndim,ndim),ekin(ndim,ndim),ndimvec(ndim))
darr3=1

print *, 'number datapoints read (jval): ',jval

! Read positions
open(31,FILE=trim(posfile),action="read", iostat=ios)
read(31, '(i)', iostat=ios) nat
read(31, '(A)', iostat=ios) strucv
print *, "nat: ", nat
print *, "strucv: ", strucv
allocate(posxyz(nat,3),possxyz(nat,3),atomLabels(nat))
i=1
do while (ios == 0.and.i.le.nat)
 read(31, '(A)', iostat=ios) puffer
 read(puffer,*, iostat=ioss)  atomLabels(i),posxyz(i,1:3)
 print *, "atomLabel, position(",i,"): ",trim(atomLabels(i)), posxyz(i,1:3)
i=i+1
end do



! Define Unit cell
i=scan(strucv,':')+1
k=1 ; j=i
do while (k.le.3)
 strucv=strucv(j:100)
 j=scan(strucv,'n')-2
 read(strucv(1:j),*, iostat=ioss)  uc(k,1:3)
 j=j+3
 k=k+1
end do
print *, "uc: ", uc(1:3,1:3)
if (nx.lt.1) then
 nx=nint(norm2(uc(1,1:3))*10.d0)
 ny=nint(norm2(uc(2,1:3))*10.d0)
 nz=nint(norm2(uc(3,1:3))*10.d0)
end if
print *,"nx,ny,nz : ",nx,ny,nz


! Read modes
! Locate vib_files
call system('ls  vib_*/vib_*.dat > fileContents.txt')
open(31,FILE='fileContents.txt',action="read")
!how many
i = 0
do
read(31,FMT='(a)',iostat=reason) r
if (reason/=0) EXIT
 i = i+1
end do
nmode=i
print *, "Number of vibrational mode files: ", nmode
 allocate(stationFileNames(nmode))
rewind(31)
do imode = 1,nmode
 read(31,'(a)') stationFileNames(imode)
end do
close(31)

allocate(pmode(nmode,nat,3),hpmode(nmode,nat,3))

!read in vib_files
imode = 1 
do while (imode.le.nmode)
 open(31,FILE=trim(stationFileNames(imode)),action="read", iostat=ios)
 if (ios /= 0) exit
 print *,"vib-file: ",trim(stationFileNames(imode))
 iat=1
 do while ((ios == 0).and.(iat.le.nat))
  read(31, '(A)', iostat=ios) puffer
  if (scan(puffer,'abcdefghijklmnopqrstuvwxyz#').eq.0.and.scan(puffer,'0123456789').gt.0) then ! test if object is a number
   read(puffer,*, iostat=ioss) pmode(imode,iat,1:3)
 !  if (trim(verbos)=='high') then 
    print *,"pmode(",imode,iat,"1:3): ",pmode(imode,iat,1:3)
  ! end if
   iat=iat+1
  end if 
 end do
 close(31)
 imode=imode+1
end do

if (trim(verbos)=='high') then 
 print *,"length mode(1,1): ",norm2(pmode(1,1,1:3)), "pmode, ",pmode(1,1,1:3) 
end if

! Read read in eigenmode koefficients
! Locate read in eigenmode koefficients
call system('ls  vib_*/eigenvecs.dat > fileContents.txt')
open(31,FILE='fileContents.txt',action="read")
!how many
i = 0
do
 read(31,FMT='(a)',iostat=reason) r
 if (reason/=0) EXIT
 i = i+1
end do
NstationFiles = i
print *, "Number of eigenvecs-files: ", NstationFiles 
rewind(31)
do i = 1,NstationFiles
 read(31,'(a)') stationFileNames(i)
end do
close(31)

! count number of lines in first file
open(31,FILE=trim(stationFileNames(1)),action="read", iostat=ios)
do
 read(31,FMT='(a)',iostat=reason) r
 if (reason/=0) EXIT
 i = i+1
end do
nstat=i-1
ncf=nstat
close(31)

allocate(cfmode(nmode,nstat,ncf),hcfmode(nmode),amode(nmode),bmode(nmode),emode(nmode,nstat),mmode(nmode,nstat),sigmode(nmode,nstat),erefmode(nmode))

!read in eigenmode koefficients
i = 1 
do while (i.le.NstationFiles)
 open(31,FILE=trim(stationFileNames(i)),action="read", iostat=ios)
 if (ios /= 0) exit
 print *,"vib-file: ",trim(stationFileNames(i))
 j=1
 do while (ios == 0)
  read(31, '(A)', iostat=ios) puffer
  if  (scan(puffer,'abcfghijklmnopqrstuvwxyz#').gt.0) then ! test if object is a number
   read(puffer,'(a)', iostat=ioss)  hcfmode(i)
   print *,"hcfmode(",i,"): ",trim(hcfmode(i))
   kk=scan(puffer(1:100000),':')
   jj=scan(puffer(kk+1:100000),'d')+1+kk
   read(puffer(kk+1:jj-1),*, iostat=ioss)   erefmode(i),amode(i),bmode(i)
   print *,"erefmode,amode,bmode(",i,"): ",erefmode(i),amode(i),bmode(i)
  else if (scan(puffer,'abcfghijklmnopqrstuvwxyz#').eq.0.and.scan(puffer,'0123456789').gt.0) then 
   k=scan(puffer,'.')-3
   puffer=puffer(k:100000)
   kk=1;k=0
   do while (k.le.3)
    kk=scan(puffer(kk:100000),'.')+kk 
   k=k+1
   end do
   kk=kk-4
   !print *,"puffer: ",trim(puffer(1:kk))
   read(puffer(1:kk),*, iostat=ioss)   emode(i,j),mmode(i,j),sigmode(i,j)
   puffer=puffer(kk:100000);k=1;kk=4
   do while (k.le.ncf.and.ioss==0.and.scan(puffer(kk+1:100000),'.').gt.0)
    jj=scan(puffer(kk+1:100000),'.')+1+kk
    !print *,"puffer(",kk-3,":",jj-4,")",trim(puffer(kk-3:jj-4))
    read(puffer(kk-3:jj-4),*, iostat=ioss)  cfmode(i,j,k)
    kk=jj ; k=k+1
   end do
   !print *,"puffer: ",trim(puffer(kk:500))
   !print *,"cfmode(",i,j,"1:nstat): ",cfmode(i,j,1:nstat)
   j=j+1
  end if 
 end do
 close(31)
 i = i+1
end do

if (trim(verbos)=='high') then 
 print *,"length mode(1,1): ",norm2(pmode(1,1,1:3)), "pmode, ",pmode(1,1,1:3) 
 print *,"emode(3,1:10): ",emode(3,1:10)
 print *,"emode(2,1),mmode(2,1),sigmode(2,1): ",emode(2,1),mmode(2,1),sigmode(2,1)
 print *,"cfmode(",2,",",10,",1:10): ",cfmode(2,10,1:10)
end if


allocate(xarr(nx,ny,nz,3),dxarr(ntemp,nx,ny,nz)) 

i=1
 do while (i.le.nx)
 j=1
  do while (j.le.ny)
   k=1
   do while (k.le.nz)
    xarr(i,j,k,1:3)=matmul(uc,(/ (i-1.d0/2)/nx,(j-1.d0/2)/ny,(k-1.d0/2)/nz /))
    if (nx*ny*nz.le.27) then
     print *,"xarr(",i,",",j,",",k,"): "
     print *, xarr(i,j,k,1:3)
    end if
    k=k+1
   end do
  j=j+1	
 end do
 i=i+1
end do


!xarr(1:nx,1:ny,1:nz,1:3)= reshape( (/( (/( (/( (i-1.d0/2)/nx*uc(1,1:3)+(j-1.d0/2)/ny*uc(2,1:3)+(k-1.d0/2)/nz*uc(3,1:3),k=1,nz )/),j=1,ny )/),i=1,nx )/),(/nx,ny,nz,3/)) 




k=1
do while (k.le.nat)
 possxyz(k,1:3)=posxyz(k,1:3)+shift
 j=1
 do while (j.le.nmode)
  possxyz(k,1:3)=possxyz(k,1:3)+(bmode(j)+amode(j))/2*pmode(j,k,1:3)
  j=j+1
 end do
 print *,"shift atom ",k,"by ",possxyz(k,1:3)-posxyz(k,1:3)," mode ",j
 k=k+1
end do

allocate(tdimarr(ntemp,nat,9),freea(ntemp))

do imode=1,nmode
  hpmode(imode,1:nat,1:3)=matmul(uc(1:3,1:3),pmode(imode,1:nat,1:3))/maxval((/ ( norm2(uc(k,1:3)) ,k=1,3) /))
  print *,imode,hpmode(imode,1:nat,1:3)
end do

tdimarr(1:ntemp,1:nat,1:9)=VibEllips(emode(1:nmode,1:nstat),mmode(1:nmode,1:nstat),sigmode(1:nmode,1:nstat), hpmode(1:nmode,1:nat,1:3),temparr(1:ntemp),ntemp,nat) 

 
itemp=1
do while (itemp.le.ntemp)
 print *,"T =", temparr(itemp)
  iat=1
  do while (iat.le.nat)
   print *,iat,tdimarr(itemp,iat,1:9)
!  details for dsyev on netlib.org
!  call DSYEV( JOBZ, UPLO, N, A, LDA, W, WORK, LWORK, INFO )
   lwork=-1
   ndimarr(1:3,1:3)=reshape(tdimarr(itemp,iat,1:9),(/ 3 , 3 /))
   call DSYEV('v','u',3,ndimarr(1:3,1:3),3,ndimvec(1:3),dvec(1:1),lwork,info)
   lwork=int(dvec(1))
   if (trim(verbos)=='high')  print *, '1. lwork: ',lwork,' info: ',info
   if (ALLOCATED(work)==.false.) allocate (work(lwork))
   call DSYEV('v','u',3,ndimarr(1:3,1:3),3,ndimvec(1:3),work(1:lwork),lwork, info)
   print *,iat,sqrt(ndimvec(1:3))
   iat=iat+1 
  end do
 itemp=itemp+1
end do


j=1
do while (j.le.nmode)
 pmode(j,1:nat,1:3)=pmode(j,1:nat,1:3)*(bmode(j)-amode(j))
 if (trim(verbos)=='high') then 
  print *,"length mode(",j,",1): ",norm2(pmode(j,1,1:3)), "pmode, ",pmode(j,1,1:3),"diff: ",(bmode(j)-amode(j))
 end if
 j=j+1
end do

j=1
if (trim(verbos)=='high') then 
 print *,"EigenVecsToFun: ",EigenVecsToFun(0.1d0,amode(j),bmode(j),emode(j,1:nstat),cfmode(j,1:nstat,1:ncf),100.d0,0)
 print *,"MeV: ",MeV((/0.1d0,0.d0 /),amode(1:2),bmode(1:2),emode(1:2,1:nstat),cfmode(1:2,1:nstat,1:ncf),100.d0,0)
 print *, "density mode 3: ", (/ (   eigenvecstofun(-1.d0+j*2.d0/16,amode(3),bmode(3),emode(3,1:nstat),cfmode(1,1:nstat,1:ncf),1.d0,0)    ,j=1,15)  /)
 print *, "MeV density mode 3: ", (/ (  MeV((/-1.d0+j*2.d0/16,0.d0,0.d0 /),amode(1:3),bmode(1:3),emode(1:3,1:nstat),cfmode(1:3,1:nstat,1:ncf),1.d0,0)    ,j=1,15)  /)
 print *, "density modes 1:3  at y=0: ", (/ (   eigenvecstofun(0.d0,amode(j),bmode(j),emode(j,1:nstat),cfmode(j,1:nstat,1:ncf),1.d0,0)    ,j=1,3)  /)


 print *,"PixToDen: ",PixToDen(xarr(2,2,2,1:3),amode,bmode,emode,cfmode,temparr,1,pmode,possxyz,xarr(1,1,1,1:3)) ! 
 print *,"PixToDen possxyz(1,1:3): ",PixToDen(possxyz(1,1:3),amode(1:nmode),bmode(1:nmode),emode(1:nmode,1:nstat),cfmode(1:nmode,1:nstat,1:ncf),temparr(1:ntemp),ntemp,pmode(1:nmode,1:nat,1:3),possxyz,xarr(1,1,1,1:3))
 print *,"PixToDen possxyz(2,1:3): ",PixToDen(possxyz(2,1:3)+(/ 1.d-2, 0.d0,0.d0/),amode(1:nmode),bmode(1:nmode),emode(1:nmode,1:nstat),cfmode(1:nmode,1:nstat,1:ncf),temparr(1:ntemp),ntemp,pmode(1:nmode,1:nat,1:3),possxyz,xarr(1,1,1,1:3))
end if ! verbos high

if (trim(verbos)=='high') then 
 i=1 ;  
 do while (i.le.30) 
   print *,"PixToDen possxyz(2,1:3): ",i,PixToDen(2*(/ 5.d-1, 5.d-1,5.d-1/)+possxyz(1,1:3)+((possxyz(2,1:3)-possxyz(1,1:3)))/16*i,amode(1:nmode),bmode(1:nmode),emode(1:nmode,1:nstat),cfmode(1:nmode,1:nstat,1:ncf),temparr(1:ntemp),ntemp,pmode(1:nmode,1:nat,1:3),possxyz,xarr(1,1,1,1:3))
   i=i+1 ;
 end do

print *,"PixToDen possxyz(2,1:3): ",PixToDen(possxyz(2,1:3)+(/ 1.d-2, 0.d0,0.d0/),amode(1:nmode),bmode(1:nmode),emode(1:nmode,1:nstat),cfmode(1:nmode,1:nstat,1:ncf),temparr(1:ntemp),ntemp,pmode(1:nmode,1:nat,1:3),possxyz,xarr(1,1,1,1:3))
end if !verbos high 



 print *,"d_erefs [eV]: ",erefmode(1:nmode)-erefmode(1)
 print *,"zero-point [eV]: ",sum(emode(1:nmode,1)-erefmode(1))

 freea=FreeE(emode(1:nmode,1:nstat),temparr(1:ntemp),ntemp)
 print *, "FreeE(:) contrib :",freea
 print *, "FreeE(:) :",erefmode(1)+sum(emode(1:nmode,1)-erefmode(1))+freea


 


! Calculate density at each pixel
dxarr=0.d0 ; i=1 ; jj=0 
do while (i.le.nx) ; j=1
 do while (j.le.ny) ; k=1
  do while (k.le.nz)
   dxarr(1:ntemp,i,j,k)=PixToDen(xarr(i,j,k,1:3),amode(1:nmode),bmode(1:nmode),emode(1:nmode,1:nstat),cfmode(1:nmode,1:nstat,1:ncf),temparr(1:ntemp),ntemp,pmode(1:nmode,1:nat,1:3),possxyz,xarr(1,1,1,1:3)) 
    if ((trim(verbos)=='high').and.norm2(xarr(i,j,k,1:3)-possxyz(2,1:3)).lt.2.d0) then 
     print *,"xarr(i,j,k,1:3) ",xarr(i,j,k,1:3)," dxarr(1:ntemp,i,j,5",k,")",dxarr(1:ntemp,i,j,k)
    end if 
   k=k+1 ; jj=jj+1
  end do
  j=j+1
 end do
 i=i+1
end do

 
if (trim(verbos)=='high'.and.nx*ny*nz.lt.30) then 
 print *,"dxarr(i,j,k) ",dxarr(1:ntemp,:,:,:)
 print *,"pos(",i,"): ",atnum('H'),atnum(' H'),atnum('H ') ,jj
end if 


allocate(work(nx*ny*nz))
! Output density to cube files
k=1 ; 
do while (k.le.ntemp)
 write(ofilename,'(i)') int(temparr(k)) 
 ofilename = trim(cubeofile) // '_' // trim(adjustl(ofilename)) // '.cube'
 print *,"ofilename : ",trim(ofilename)
 open(fg, file=ofilename)
 write(fg,"('-Quickstep-',/,'  Temp [K]= ',1x,F18.8,/,i5,' 0 0 0')") temparr(k),nat
 write(fg,"(i5,3F18.8,1x,/,i5,3F18.8,1x,/,i5,3F18.8,1x)") nx,uc(1,1:3)/nx,ny,uc(2,1:3)/ny,nz,uc(3,1:3)/nz
 i=1 
 do while (i.le.nat)
 ! print *,"pos(",i,"): ",atnum(trim(atomLabels(i))),possxyz(i,1:3)
  write(fg,"(i5,' 0 ',F18.8,1x,F18.8,1x,F18.8,1x)") atnum(trim(atomLabels(i))),possxyz(i,1:3)
 i=i+1
 end do
!-Quickstep-
!    WFN 00053_1     0.0000
!2 0 0 0
! 30    0.33333    0    0  
! 30    0    0.33333    0  
! 30    0    0    0.33333  
!  1    0    5.15495    5.15495    5.15495  
!  1    0    5.58885    5.58885    5.58885
 work=reshape(dxarr(k,1:nx,1:ny,1:nz)/maxval(dxarr(k,1:nx,1:ny,1:nz)),(/ nx*ny*nz /))
  i=1 
 do while (i.lt.nx*ny*nz-6)
  j=i+5
  write(fg,"(1x,E18.8,1x,E18.8,1x,E18.8,1x,E18.8,1x,E18.8,1x,E18.8,1x)") work(i:j) !
  i=i+6
 end do
 write(fg,"(1x,E18.8,1x,E18.8,1x,E18.8,1x,E18.8,1x,E18.8,1x,E18.8,1x)") work(i-6:nx*ny*nz)
 close(fg)
 k=k+1
end do !ntemp


deallocate(work,posxyz,possxyz,atomLabels,stationFileNames,pmode,cfmode,hcfmode,amode,bmode,emode,mmode,sigmode,erefmode,xarr,dxarr)
call system('rm fileContents.txt')

stop

! convert angstroms to bohr and ryd to hartree
ipd(1:jval,1)=ipd(1:jval,1)/bohrangst
ipd(1:jval,3)=ipd(1:jval,3)/2.d0


call hpsort(jval, ipd(1:jval,1), vector)
ipd(1:jval,2)=ipd(vector(1:jval),2)
rvec(1:jval)=ipd(1:jval,3)
call hpsort(jval,rvec(1:jval),ivec)
eref=ipd(ivec(1),3)
daphon(1,1:3)=ipd(ivec(1:3),1)
daphon(2,1:3)=ipd(ivec(1:3),3)
ipd(1:jval,3)=ipd(vector(1:jval),3)-eref
ivec =0
call hpsort(3,daphon(1,1:3),ivec(1:3)) ! sort amplitudes
daphon(2,1:3)=daphon(2,ivec(1:3)) ! sort enerergies
if (trim(verbos)=='high') print *, 'daphon1: ', daphon(1,1:3)
if (trim(verbos)=='high') print *, 'daphon2: ', daphon(2,1:3)

! Resonance frequence from derivative at minimum
daphon(1,1)=daphon(1,2)-daphon(1,1)
daphon(1,3)=daphon(1,3)-daphon(1,2)
daphon(1,2)=-daphon(1,1)-daphon(1,3) 
daphon(2,1)=sqrt((-1.d0)*dot_product(daphon(1,1:3),daphon(2,1:3))/daphon(1,1)/daphon(1,2)/daphon(1,3)*2/mass)*hartev

if ((trim(verbos)=='high').or.(trim(verbos)=='mid'))  then 
 write(*,"(' phonon (harm_app) Hz, cm-1: ',E18.8,1x,1x,6F14.5,1x)") daphon(2,1)*evj*(/ 1.d0/hplanck,1.d-2/(hplanck*clight) /)
end if 

i=1   ! drop energies which are too high 
do while (i<=jval) 
 if (trim(verbos)=='high') write(*,"(1x,'ipd(',I4,'): ', F18.10,1x,I5,1x,F18.10)") i,ipd(i,1),int(ipd(i,2)),ipd(i,3)
 if (ipd(i,3).gt.maxen/2.d0) then
  ipd(i:jval-1,:)=ipd(i+1:jval,:)
  jval=jval-1
 else
  i=i+1
 end if
end do
print *, 'number datapoints used: ',jval


!Find boundaries
nkt=2 ! number of points used at the boundary nd > nkt > 2 

bas(1:nkt,1)=1.d0
bas(1:nkt,2)=1.d0/ipd(1:nkt,1)
bas(1:2,1)=linearfit(reshape(1.d0/ipd(1:nkt,3),(/nkt/)),bas(1:nkt,1:2))
a=-bas(2,1)/bas(1,1)
if ((a.lt.1.5*ipd(1,1)).or.(a/ipd(1,1).le.0)) then
  print *, 'WARNING: lower boundary found at: ',a*bohrangst
  a=1.2*ipd(1,1)
  print *, 'WARNING: lower boundary reset to: ',a*bohrangst
else if (trim(verbos)=='high') then
  print *,' lower boundary at: ',a*bohrangst
end if

bas(1:nkt,1)=1.d0
bas(1:nkt,2)=1/ipd(jval-nkt+1:jval,1)
bas(1:2,1)=linearfit(reshape(1.d0/ipd(jval-nkt+1:jval,3),(/nkt/)),bas(1:nkt,1:2))
b=-bas(2,1)/bas(1,1)
if ((b.gt.1.5*ipd(jval,1)).or.(b/ipd(jval,1).le.0)) then
  print *, 'WARNING: upper boundary found at: ',b*bohrangst
  b=1.2*ipd(jval,1)
  print *, 'WARNING: upper boundary reset to: ',b*bohrangst
else if (trim(verbos)=='high') then
  print *,' upper boundary at: ',b*bohrangst
end if

if (abs(ymin).gt.10d-10.or.abs(ymax).gt.10d-10) then
 print *, 'freymin, freymax: ',freymin,freymax
 if (abs(ymin).gt.10d-10) a=ymin
 if (abs(ymax).gt.10d-10) b=ymax
 if (freymin.eq.0.and.abs(ymax).gt.10d-10) a=-ymax
 if (freymax.eq.0.and.abs(ymin).gt.10d-10) b=-ymin
 print *, 'WARNING: boundaries provided, a: ',a*bohrangst,' and b: ',b*bohrangst,' [angst.]'
end if

!amplitudes mapped on 0,1
dbasa=xtoy(ipd(1:jval,1),a,b)
if (trim(verbos)=='high') print *, 'dbasa: ', dbasa(1:jval)

!amplitude of confining potential 1/(1-y^2)
darrn(1:2*nkt,1)=reshape((/ dbasa(1:nkt),dbasa(jval-nkt+1:jval) /),(/2*nkt/))
darrn(1:2*nkt,2)=reshape((/ ipd(1:nkt,3),ipd(jval-nkt+1:jval,3) /),(/2*nkt/))
print *, 'darrn(1:2*nkt,1): ',darrn(1:2*nkt,1)
print *, 'darrn(1:2*nkt,2): ',darrn(1:2*nkt,2)
cp=dot_product(darrn(1:2*nkt,2),darrn(1:2*nkt,2))/dot_product(darrn(1:2*nkt,2),ytoconfp(darrn(1:2*nkt,1))) 
darrn(1:2*nkt,1)=ytoconfp(darrn(1:2*nkt,1))
print *, 'ytoconfp: ',darrn(1:2*nkt,1)

if (trim(verbos)=='high') print *, '1/(1-y^2)-1 contribution: ',cp

!decomposition into harmonic functions Sin[ j Pi/2 (x + 1)] 
!frq=(/ (i*dpi, i = 1, ndim) /)
!if (trim(verbos)=='high') print *,'frq: ',frq(1:ndim)
rvec(1:jval)=ipd(1:jval,3)-cp*ytoconfp(dbasa(1:jval))

if (trim(verbos)=='high') print *, 'rvec: ',rvec(1:jval)

np=jval ! number of points in the fit
if (np.gt.jval ) rvec(jval+1:np)=0
i=1 
nb=jval-2 ! number basis vectors
do while (i<=nb)
 ! bas(1:np,i)=reshape((/ dbasa(1:jval),xtoy((/ a+(ipd(1,1)-a)/2, b+(ipd(jval,1)-b)/2 /),a,b) /),(/ np /))**i
 ! bas(1:np,i)=reshape((/ dbasa(1:jval),xtoy((/ a+(ipd(1,1)-a)/2, b+(ipd(jval,1)-b)/2 /),a,b) /),(/ np /))
 bas(1:np,i)=dbasa(1:jval)
 if ((trim(verbos)=='high').and.(i==1)) print *, 'bas(1:np,1): ',bas(1:np,i)
 bas(1:np,i)=sin(i*dpi/2 *(bas(1:np,i)+1.0D0))
 if ((trim(verbos)=='high').and.(i.le.3)) print *, 'bas(1:np,',i,'): ',bas(1:np,i)
 i=i+1
end do
if (trim(verbos)=='high') print *, 'size(bas) : ', size(bas)


harc(1:jval)=linearfit(rvec(1:np),bas(1:np,1:nb))
if (trim(verbos)=='high') print *, 'harmonic components: ',harc(1:nb)

! Check Romberg Algorythem

if (trim(verbos)=='high') then
 !call rombe (func,b,a,aserr,relerr,error,result,ind)
 !call rombe (func,b,a,1.d-15,1.d-15,darrn(1,3),darrn(1,4),i)
 !print *, 'error: ', darrn(1,3)
 !print *, 'int_a^b y^2 dy: ', darrn(1,4)
 !print *, 'ind: ', i
 i=1
 do while (i<=6) 
  call cisi(dpi*i,ci,si)
  rvecs(i)=ci
  i=i+1
 end do
 print *, 'CosIntegral[{1,2,3,4,5,6}* Pi] ', rvecs(1:6)
 print *, 'CI[4 Pi],-CI[6], Log[3/2] ', rvecs(4),rvecs(6),dlog((1+5)/dble(iabs(1-5)))
 print *, 'CI[4 Pi],-CI[6], Log[3/2] ', rvecs(4),rvecs(6),dlog((5+1)/dble(iabs(5-1)))
end if


! Loop on k-points, only for band structure  
mdim=4 !INT((ndim/10)+1) ! number of bands plotted for bandstructure
do while (jkpoint<=nkpoint)
 kpointvec(jkpoint)=(jkpoint-1)/(nkpoint-1d0+1.d-12)
! Setup Hamiltonian

w=dpi/(b-a)
if (trim(verbos)=='high') then ! Tests fro naturalsplin and integration 
bas=0.d0
bas(1:6,1)=(/  -10, -6, -2, 2, 6, 10  /)
bas(1:6,2)=(/  100, 36, 4, 4, 36, 100  /)
bas(7:11,1:6)=naturalsplin(bas(1:6,1:2))
print *, 'bas(7,1:6): ',bas(7,1:6)
print *, 'splin(2,00001',splin(2.0000001d0,bas(7:11,1:6))
print *, 'splin(10.01',splin(10.01d0,bas(7:11,1:6))
print *, 'ka(3,5,-2,1d0,2.1d0)',ka(3,5.d0,-2.d0,1d0,2.1d0)
print *, 'li(3,2,4,-2,5,1,2.1)',li(3,2,4,-2.d0,5d0,1d0,2.1d0)
print *, 'ish(bas(7,1:6),bas(7,5),bas(7,6),,2,4,-10,10)',ish(bas(7,1:6),bas(7,5),bas(7,6),2,4,-10d0,10d0)
print *, 'is(bas(7:11,1:6),2,4,-10,10)',is(bas(7:11,1:6),2,4,-10d0,10d0)
end if

! Calculation of spline coefficients

bas(1:jval,1)=dbasa(1:jval)
bas(1:jval,2)=rvec(1:jval) 
print *, 'dbasa(1:jval)',bas(1:jval,1)
print *, 'rvec(1:jval) ',bas(1:jval,2)
splinc(1:jval-1,1:6)=naturalsplin(bas(1:jval,1:2))
if (trim(verbos)=='high') print *, 'spline coefficients: ',splinc(1:jval-1,2)


i=1
do while (i<=ndim) 
 j=i
 do while (j<=ndim) 
  !Add contribution due to harmonic component
  ! if (trim(verbos)=='high') print *, 'Jis(i,j,k): ',    (/(Jis(i,j,k),k=1,jval)/)
   if (trim(mode)=='harmonic') epot(i,j)=sum( (/(Kis(i,j,k)*harc(k),k=1,nb)/) )
   if (trim(mode)=='spline') epot(i,j)=is(splinc(1:jval-1,1:6),i,j,splinc(1,5),splinc(jval-1,6))
   if (trim(verbos)=='high') then
     print *, 'harmonic contrib:',i,j,sum( (/(Kis(i,j,k)*harc(k),k=1,nb)/) )
     print *, 'spline contrib:',i,j, is(splinc(1:jval-1,1:6),i,j,splinc(1,5),splinc(jval-1,6))
   end if ! verbos == high
  !Add contribution due to confining potential   
   if (i==j)  then
    epot(i,j)=epot(i,j)+cp*(Cfp(i,j)-1.d0)
    ekin(i,j)=(j**2+kpointvec(jkpoint)**2)*(w**2/(2*mass))
   else
    epot(i,j)=epot(i,j)+cp*Cfp(i,j)+(4*((-1)**(j+i)-1)*(b-a)*j*i)/((j**2-i**2)**2*dpi**2)*vpert
    ekin(i,j)=0.d0
   end if
   if (i/=j)  epot(j,i)=epot(i,j)
  j=j+1
 ! if (j>4) exit
 end do
 ndimarr(1:ndim,1:ndim)=epot(1:ndim,1:ndim)+ekin(1:ndim,1:ndim)

 if (trim(verbos)=='high') then
  ! print *, 'Cfp(6,All): ',    (/ ((Cfp(j,k),k=1,12),j=1,12)/)
  print *, 'epot ',i,' All ',epot(i,1:ndim)
  print *, 'ekin ',i,ekin(i,i)
 end if ! verbos == high
 ! exit
 i=i+1
end do

! Diagonalization of Hamiltonian

lwork=-1
! details for dsyev on netlib.org
! call DSYEV( JOBZ, UPLO, N, A, LDA, W, WORK, LWORK, INFO )
  call DSYEV('v','u',ndim,darr2,ndim,ndimvec(1:ndim),dvec(1:1),lwork,info )
  lwork=int(dvec(1))
if (trim(verbos)=='high')  print *, '1. lwork: ',lwork,' info: ',info
  if (ALLOCATED(work)==.false.) allocate (work(lwork))
  call DSYEV('v','u',ndim,ndimarr,ndim,ndimvec(1:ndim),work,lwork, info )

rvec(1:5)=ndimvec(1:5)*hartev
dvec(1:2)=(rvec(2)-rvec(1))*evj*(/ 1.d0/hplanck,1.d-2/(hplanck*clight) /) 
dvec(3:4)=(rvec(3)-rvec(2))*evj*(/ 1.d0/hplanck,1.d-2/(hplanck*clight) /) 

 ekpoints(jkpoint,1:mdim)=(ndimvec(1:mdim)+eref)*hartev
 jkpoint=jkpoint+1
end do ! Loop on k-points, only for band structure 
 

call DATE_AND_TIME(date, time)

! Output to screen
 write(*, "(' date: ',a8,2x,'time: ',a10)") trim(date),trim(time)

if ((trim(verbos)=='high').or.(trim(verbos)=='mid'))  then 
 write(*,"(' 2. lwork: ',F8.3,1x,' info: ',I4,1x,'lowest eigenvalues in eV: ',6F14.5,1x)") work(1:1),info,rvec(1:5)
 write(*,"(' resonance frequency in Hz (cm^-1): ',E18.8,1x,F18.8,1x,' 2nd res Hz (cm^-1): ',E18.8,1x,F18.8)") dvec(1:4)
 write(*,"(' zero point energy energy in cm^-1: ',F18.8,1x)") rvec(1)*evj*1.d-2/(hplanck*clight)
 write(*,"(' case harmonic oscillator proportions (1,3,5,7,9): ',6F14.5,1x)")rvec(1:5)/rvec(1)
end if 

if ((int(nkpoint).gt.1))  then ! Output and stop for band structure
 open(fg, file=outfile)
 write(fg,"('#kpt (pi/a), energy (eV);  pot[Ha]:',F18.8,1x,'date: ',a8,2x,'time: ',a10)")eref,trim(date),trim(time)
 write(my_fmt, '(a, i0, a)') '(1x,I4,', mdim+1, '(1x,F18.8))' ! scientific format: E18.11
 jkpoint=1 
 do while (jkpoint<=nkpoint)
  write(fg,my_fmt) jkpoint,kpointvec(jkpoint),ekpoints(jkpoint,1:mdim)
  jkpoint=jkpoint+1
 end do
 close(fg)
 deallocate (hhndimarr,hndimarr,ndimarr,ndimvec,work,epot,ekin)
 call exit(0)
end if



if (ndimvec(1).lt.0)  then 
 print *, 'WARNING: eigenstates with negative energies'
end if 

! Convert to Rydberg
ndimvec(:)=ndimvec(:)*2
eref=eref*2

! Output coefficients of eigenfunctions to file
 open(fg, file=vectorout)
 write(my_fmt, '(a, i0, a)') '(1x,I4,1x,E20.11,1x,E20.11,1x,E20.11,1x,', ndim, '(1x,E18.11))'
 write(fg,"('#inx, E[Ry],mean,sigma^2, coeffs Sin[i*Pi/2*(x + 1)], a[angst],b[angst]; E_ref[Ry]:',3F18.8,1x,'date: ',a8,2x,'time: ',a10)") eref,a,b,trim(date),trim(time)
 i=1
 do while (i<=ndim) ! setup matrices for first and second moment 
  j=i
  do while (j<=ndim)
   if (i.eq.j) then
    hndimarr(i,j)=0.0d0
    hhndimarr(i,j)=(2*j**2*dpi**2*(a**2+a*b+b**2)-3*(a-b)**2)/(6*j**2*dpi**2)
   else
    hndimarr(i,j)=(4*((-1)**(j+i)-1)*(b-a)*j*i)/((j**2-i**2)**2*dpi**2)
    hhndimarr(i,j)=(8*(a-b)*j*i*(a-b*(-1)**(j+i)))/((j**2-i**2)**2*dpi**2)
   end if
   hndimarr(j,i)=hndimarr(i,j)
   hhndimarr(j,i)=hhndimarr(i,j)
   j=j+1
  end do
  if ((trim(verbos)=='high').and.(i.lt.4)) then
   print *, 'hndimarr ',i,' All ',(/ (hndimarr(i,j) ,j=1,4)/)
   print *, 'hhndimarr ',i,' All ',(/ (hhndimarr(i,j),j=1,4)/)
  end if ! verbos == high
  i=i+1
 end do 
 hndimarr=MATMUL(hndimarr,ndimarr)
 hhndimarr=MATMUL(hhndimarr,ndimarr)
 i=1
 do while (i<ndim)
  meann=DOT_PRODUCT(ndimarr(:,i),hndimarr(:,i)) ! mean
  dbasa(1:1)=DOT_PRODUCT(ndimarr(:,i),hhndimarr(:,i))-meann**2 ! sigma**2
  write(fg,my_fmt) i,ndimvec(i)+eref,meann,dbasa(1:1),ndimarr(:,i)
 i=i+1
 end do
 close(fg)

! Output eigenvalues to file
 open(fg, file=outfile)
 i=1
 write(fg,"('#inx, x, pot[Ha]:',F18.8,1x,'date: ',a8,2x,'time: ',a10)") eref,trim(date),trim(time)
 write(fg,"('#v_res [Hz], [cm^-1]',1x,E18.8,1x,F18.5,1x,'v2_res [Hz], [cm^-1]',1x,E18.8,1x,F18.5,1x)") dvec(1:4)
 do while (i<=ndim)
  write(fg,"(1x,I4,1x,F18.10,1x)") i,(ndimvec(i)+eref) 
 i=i+1
 end do
 close(fg)  

! Calculate Ekin, Epot separately for lowest states
epot(1:ndim,1:ndim) = MATMUL(epot(1:ndim,1:ndim),ndimarr(1:ndim,1:ndim))
epot(1:ndim,1:ndim) = MATMUL(transpose(ndimarr(1:ndim,1:ndim)), epot(1:ndim,1:ndim))*hartev
ekin(1:ndim,1:ndim) = MATMUL(ekin(1:ndim,1:ndim),ndimarr(1:ndim,1:ndim))
ekin(1:ndim,1:ndim) = MATMUL(transpose(ndimarr(1:ndim,1:ndim)), ekin(1:ndim,1:ndim))  *hartev 
write(*,"(' Epot,Ekin, state 1 [eV]: ',F18.8,1x,F18.8,1x,F18.8,1x,F18.8,1x,F18.8,1x)") epot(1,1),ekin(1,1)
write(*,"(' Epot,Ekin, state 2 [eV]: ',F18.8,1x,F18.8,1x,F18.8,1x,F18.8,1x,F18.8,1x)") epot(2,2),ekin(2,2)
write(*,"(' Epot,Ekin, state 3 [eV]: ',F18.8,1x,F18.8,1x,F18.8,1x,F18.8,1x,F18.8,1x)") epot(3,3),ekin(3,3)
write(*,"(' Epot,Ekin, state 4 [eV]: ',F18.8,1x,F18.8,1x,F18.8,1x,F18.8,1x,F18.8,1x)") epot(4,4),ekin(4,4)
write(*,"(' Epot,Ekin, state 5 [eV]: ',F18.8,1x,F18.8,1x,F18.8,1x,F18.8,1x,F18.8,1x)") epot(5,5),ekin(5,5)

 


deallocate (hhndimarr,hndimarr,ndimarr,ndimvec,work,epot,ekin)


end program control_file
!amp j energ[Ry] mass[m_e]
!mass  123681.48839917
!0 0 -1189.44071304
!.62573784 1 -1189.45293046
