program control_file
  use eigenstate_utils
  use operations
 
  implicit none

  ! Input related variables
  character(len=10000000) :: my_fmt
  character(len=100) :: buffer, puffer, label, outfile, vectorout,verbos ,fifile,date, time, mode
  integer :: apos,pos, jval, ndim,mdim,i,j,k,jj,kk,nkt,np,nb   
  integer, parameter :: fh = 15
  integer, parameter :: fg = 6
  integer, parameter :: ff = 7
  integer, parameter :: dm = 500 ! maximum lines that can be read
  integer     :: ios =0,ioss=0,line=0,lwork,info,freymin,freymax 
  integer     :: nkpoint=1 , jkpoint=1
!  integer, dimension(3) :: today, now

  ! Control file variables
  real :: pill,  maxen , meann , vpert=0.0d0
  real*8  , dimension(dm,dm) :: rarrn, rbrrn
  double precision :: mass, cp,hh ,hg, a, b,eref,ci,si,ymin ,ymax, w ,kpoint
  integer, dimension(dm) :: vector, ivec
  double precision , dimension(2,3) :: daphon
  double precision , dimension(dm,3) :: ipd
  double precision , dimension(dm) :: rvec,rvecs,dvec, dbasa, frq, kpointvec, harc=0.d0 
  double precision , dimension(dm,6) :: splinc=0.d0
  double precision , dimension(2,2) :: darr2, dbrr2
  double precision , dimension(3,3) :: darr3, dbrr3
  double precision , dimension(dm,dm) :: bas, darrn, ddrrn,ekpoints
  double precision , dimension(:), allocatable :: ndimvec,work
  double precision , dimension(:,:), allocatable :: ndimarr,epot,ekin,hndimarr,hhndimarr

  open(fh, file='naliz.dat')

  ndim=100 !default value of number of basis functions
  outfile='eigenstates.dat'
  vectorout='eigenvecs.dat'
  verbos='low'
  mode='harmonic'
  fifile='fit.dat'
  maxen=10.0d10
  freymax=0
  freymin=0

  ! ios is negative if an end of record condition is encountered or if
  ! an endfile condition was detected.  It is positive if an error was
  ! detected.  ios is zero otherwise.

  if (ios /= 0) then
    print *, 'Input naliz.dat does not exist'
    stop
  end if
  
  jval=1 ; ipd=0.0000 ; ymax=0.d0
  do while (ios == 0)
     read(fh, '(A)', iostat=ios) puffer
     if (ios == 0) then
        line = line + 1
        ! Find the first instance of whitespace.  Split label and data.
        apos = scan(puffer,"abcdefghijklmnopqrstuvwxyz-123456789")
        pos = scan(puffer(apos:), ' ')
        label = puffer(apos:pos+apos-1)
        buffer = puffer(apos+pos:)
   !       write(*,*) scan(puffer,"abcdefghijklmnopqrstuvwxyz-123456789") ,label
        select case (label)
        case ('mode')
           read(buffer, *, iostat=ios) mode ! harmonic, spline
           print *, 'Read verbos: ', mode
        case ('verbos')
           read(buffer, *, iostat=ios) verbos ! mid high low
           print *, 'Read verbos: ', verbos
        case ('ndim')
           read(buffer, *, iostat=ios) ndim
           print *, 'Read ndim: ', ndim
        case ('vectorout')
           read(buffer, *, iostat=ios) vectorout
           print *, 'Read vectorout: ', vectorout
        case ('outfile')
           read(buffer, *, iostat=ios) outfile
           print *, 'Read outfile: ', outfile
        case ('maxen')
           read(buffer, *, iostat=ios) maxen ! in rydberg
           print *, 'Read maxen: ', maxen
        case ('pill')
           read(buffer, *, iostat=ios) pill
           print *, 'Read pill: ', pill
        case ('ymin')
            if  (scan(buffer,'aeioud#f').eq.1) then ! test if buffer is a string
             print *,' ' 
           else
            read(buffer, *, iostat=ios) ymin ! in bohr
            print *, 'Read ymin: ', ymin,' (',ymin/bohrangst,' bohr)' 
            ymin=ymin/bohrangst
             freymin=1
           end if
        case ('ymax')
            if  (scan(buffer,'aeioud#f').eq.1) then ! test if buffer is a string
             print *,' ' 
           else
             read(buffer, *, iostat=ios) ymax ! in bohr
             print *, 'Read ymax: ', ymax,' (',ymax/bohrangst,' bohr)' 
             ymax=ymax/bohrangst
             freymax=1
           end if
        case ('vpert')
           read(buffer, *, iostat=ios) vpert ! in rydberg
           print *, 'Read vpert: ', vpert
        case ('mass')
           read(buffer, *, iostat=ios) mass ! in units of electron mass
           print *, 'Read mass: ', mass
        case ('nkpoint')
           read(buffer, *, iostat=ios) nkpoint ! number of kpoints to be calculated 
           print *, 'Read nkpoint: ', nkpoint
        case ('vector')
           read(buffer, *, iostat=ios) vector
           print *, 'Read vector: ', vector
        case default
           read(puffer, *, iostat=ioss) ipd(jval,:) 
           if  (scan(label,'abcdefghijklmnopqrstuvwxyz#').eq.0) then ! test if object is a number
            write(*,"(1x,'ipd(',I4,'): ', F18.10,1x,I5,1x,F18.10)") jval,ipd(jval,1),int(ipd(jval,2)),ipd(jval,3)
            jval=jval+1
           else
            print *, 'Skipping invalid label at line', line
           end if
        end select
     end if
  end do
jval=jval-1

allocate (hhndimarr(ndim,ndim),hndimarr(ndim,ndim),ndimarr(ndim,ndim),epot(ndim,ndim),ekin(ndim,ndim),ndimvec(ndim))
darr3=1

print *, 'number datapoints read (jval): ',jval

! convert angstroms to bohr and ryd to hartree
ipd(1:jval,1)=ipd(1:jval,1)/bohrangst
ipd(1:jval,3)=ipd(1:jval,3)/2.d0


call hpsort(jval, ipd(1:jval,1), vector)
ipd(1:jval,2)=ipd(vector(1:jval),2)
rvec(1:jval)=ipd(1:jval,3)
call hpsort(jval,rvec(1:jval),ivec)
eref=ipd(ivec(1),3)
daphon(1,1:3)=ipd(ivec(1:3),1)
daphon(2,1:3)=ipd(ivec(1:3),3)
ipd(1:jval,3)=ipd(vector(1:jval),3)-eref
ivec =0
call hpsort(3,daphon(1,1:3),ivec(1:3)) ! sort amplitudes
daphon(2,1:3)=daphon(2,ivec(1:3)) ! sort enerergies
if (trim(verbos)=='high') print *, 'daphon1: ', daphon(1,1:3)
if (trim(verbos)=='high') print *, 'daphon2: ', daphon(2,1:3)

! Resonance frequence from derivative at minimum
daphon(1,1)=daphon(1,2)-daphon(1,1)
daphon(1,3)=daphon(1,3)-daphon(1,2)
daphon(1,2)=-daphon(1,1)-daphon(1,3) 
daphon(2,1)=sqrt((-1.d0)*dot_product(daphon(1,1:3),daphon(2,1:3))/daphon(1,1)/daphon(1,2)/daphon(1,3)*2/mass)*hartev

if ((trim(verbos)=='high').or.(trim(verbos)=='mid'))  then 
 write(*,"(' phonon (harm_app) Hz, cm-1: ',E18.8,1x,1x,6F14.5,1x)") daphon(2,1)*evj*(/ 1.d0/hplanck,1.d-2/(hplanck*clight) /)
end if 

i=1   ! drop energies which are too high 
do while (i<=jval) 
 if (trim(verbos)=='high') write(*,"(1x,'ipd(',I4,'): ', F18.10,1x,I5,1x,F18.10)") i,ipd(i,1),int(ipd(i,2)),ipd(i,3)
 if (ipd(i,3).gt.maxen/2.d0) then
  ipd(i:jval-1,:)=ipd(i+1:jval,:)
  jval=jval-1
 else
  i=i+1
 end if
end do
print *, 'number datapoints used: ',jval


!Find boundaries
nkt=2 ! number of points used at the boundary nd > nkt > 2 

bas(1:nkt,1)=1.d0
bas(1:nkt,2)=1.d0/ipd(1:nkt,1)
bas(1:2,1)=linearfit(reshape(1.d0/ipd(1:nkt,3),(/nkt/)),bas(1:nkt,1:2))
a=-bas(2,1)/bas(1,1)
if ((a.lt.1.5*ipd(1,1)).or.(a/ipd(1,1).le.0)) then
  print *, 'WARNING: lower boundary found at: ',a*bohrangst
  a=1.2*ipd(1,1)
  print *, 'WARNING: lower boundary reset to: ',a*bohrangst
else if (trim(verbos)=='high') then
  print *,' lower boundary at: ',a*bohrangst
end if

bas(1:nkt,1)=1.d0
bas(1:nkt,2)=1/ipd(jval-nkt+1:jval,1)
bas(1:2,1)=linearfit(reshape(1.d0/ipd(jval-nkt+1:jval,3),(/nkt/)),bas(1:nkt,1:2))
b=-bas(2,1)/bas(1,1)
if ((b.gt.1.5*ipd(jval,1)).or.(b/ipd(jval,1).le.0)) then
  print *, 'WARNING: upper boundary found at: ',b*bohrangst
  b=1.2*ipd(jval,1)
  print *, 'WARNING: upper boundary reset to: ',b*bohrangst
else if (trim(verbos)=='high') then
  print *,' upper boundary at: ',b*bohrangst
end if

if (abs(ymin).gt.10d-10.or.abs(ymax).gt.10d-10) then
 print *, 'freymin, freymax: ',freymin,freymax
 if (abs(ymin).gt.10d-10) a=ymin
 if (abs(ymax).gt.10d-10) b=ymax
 if (freymin.eq.0.and.abs(ymax).gt.10d-10) a=-ymax
 if (freymax.eq.0.and.abs(ymin).gt.10d-10) b=-ymin
 print *, 'WARNING: boundaries provided, a: ',a*bohrangst,' and b: ',b*bohrangst,' [angst.]'
end if

!amplitudes mapped on 0,1
dbasa=xtoy(ipd(1:jval,1),a,b)
if (trim(verbos)=='high') print *, 'dbasa: ', dbasa(1:jval)

!amplitude of confining potential 1/(1-y^2)
darrn(1:2*nkt,1)=reshape((/ dbasa(1:nkt),dbasa(jval-nkt+1:jval) /),(/2*nkt/))
darrn(1:2*nkt,2)=reshape((/ ipd(1:nkt,3),ipd(jval-nkt+1:jval,3) /),(/2*nkt/))
print *, 'darrn(1:2*nkt,1): ',darrn(1:2*nkt,1)
print *, 'darrn(1:2*nkt,2): ',darrn(1:2*nkt,2)
cp=dot_product(darrn(1:2*nkt,2),darrn(1:2*nkt,2))/dot_product(darrn(1:2*nkt,2),ytoconfp(darrn(1:2*nkt,1))) 
darrn(1:2*nkt,1)=ytoconfp(darrn(1:2*nkt,1))
print *, 'ytoconfp: ',darrn(1:2*nkt,1)

if (trim(verbos)=='high') print *, '1/(1-y^2)-1 contribution: ',cp

!decomposition into harmonic functions Sin[ j Pi/2 (x + 1)] 
!frq=(/ (i*dpi, i = 1, ndim) /)
!if (trim(verbos)=='high') print *,'frq: ',frq(1:ndim)
rvec(1:jval)=ipd(1:jval,3)-cp*ytoconfp(dbasa(1:jval))

if (trim(verbos)=='high') print *, 'rvec: ',rvec(1:jval)

np=jval ! number of points in the fit
if (np.gt.jval ) rvec(jval+1:np)=0
i=1 
nb=jval-2 ! number basis vectors
do while (i<=nb)
 ! bas(1:np,i)=reshape((/ dbasa(1:jval),xtoy((/ a+(ipd(1,1)-a)/2, b+(ipd(jval,1)-b)/2 /),a,b) /),(/ np /))**i
 ! bas(1:np,i)=reshape((/ dbasa(1:jval),xtoy((/ a+(ipd(1,1)-a)/2, b+(ipd(jval,1)-b)/2 /),a,b) /),(/ np /))
 bas(1:np,i)=dbasa(1:jval)
 if ((trim(verbos)=='high').and.(i==1)) print *, 'bas(1:np,1): ',bas(1:np,i)
 bas(1:np,i)=sin(i*dpi/2 *(bas(1:np,i)+1.0D0))
 if ((trim(verbos)=='high').and.(i.le.3)) print *, 'bas(1:np,',i,'): ',bas(1:np,i)
 i=i+1
end do
if (trim(verbos)=='high') print *, 'size(bas) : ', size(bas)


harc(1:jval)=linearfit(rvec(1:np),bas(1:np,1:nb))
if (trim(verbos)=='high') print *, 'harmonic components: ',harc(1:nb)

! Check Romberg Algorythem

if (trim(verbos)=='high') then
 !call rombe (func,b,a,aserr,relerr,error,result,ind)
 !call rombe (func,b,a,1.d-15,1.d-15,darrn(1,3),darrn(1,4),i)
 !print *, 'error: ', darrn(1,3)
 !print *, 'int_a^b y^2 dy: ', darrn(1,4)
 !print *, 'ind: ', i
 i=1
 do while (i<=6) 
  call cisi(dpi*i,ci,si)
  rvecs(i)=ci
  i=i+1
 end do
 print *, 'CosIntegral[{1,2,3,4,5,6}* Pi] ', rvecs(1:6)
 print *, 'CI[4 Pi],-CI[6], Log[3/2] ', rvecs(4),rvecs(6),dlog((1+5)/dble(iabs(1-5)))
 print *, 'CI[4 Pi],-CI[6], Log[3/2] ', rvecs(4),rvecs(6),dlog((5+1)/dble(iabs(5-1)))
end if


! Loop on k-points, only for band structure  
mdim=4 !INT((ndim/10)+1) ! number of bands plotted for bandstructure
do while (jkpoint<=nkpoint)
 kpointvec(jkpoint)=(jkpoint-1)/(nkpoint-1d0+1.d-12)
! Setup Hamiltonian

w=dpi/(b-a)
if (trim(verbos)=='high') then ! Tests fro naturalsplin and integration 
bas=0.d0
bas(1:6,1)=(/  -10, -6, -2, 2, 6, 10  /)
bas(1:6,2)=(/  100, 36, 4, 4, 36, 100  /)
bas(7:11,1:6)=naturalsplin(bas(1:6,1:2))
print *, 'bas(7,1:6): ',bas(7,1:6)
print *, 'splin(2,00001',splin(2.0000001d0,bas(7:11,1:6))
print *, 'splin(10.01',splin(10.01d0,bas(7:11,1:6))
print *, 'ka(3,5,-2,1d0,2.1d0)',ka(3,5.d0,-2.d0,1d0,2.1d0)
print *, 'li(3,2,4,-2,5,1,2.1)',li(3,2,4,-2.d0,5d0,1d0,2.1d0)
print *, 'ish(bas(7,1:6),bas(7,5),bas(7,6),,2,4,-10,10)',ish(bas(7,1:6),bas(7,5),bas(7,6),2,4,-10d0,10d0)
print *, 'is(bas(7:11,1:6),2,4,-10,10)',is(bas(7:11,1:6),2,4,-10d0,10d0)
end if

! Calculation of spline coefficients

bas(1:jval,1)=dbasa(1:jval)
bas(1:jval,2)=rvec(1:jval) 
print *, 'dbasa(1:jval)',bas(1:jval,1)
print *, 'rvec(1:jval) ',bas(1:jval,2)
splinc(1:jval-1,1:6)=naturalsplin(bas(1:jval,1:2))
if (trim(verbos)=='high') print *, 'spline coefficients: ',splinc(1:jval-1,2)


i=1
do while (i<=ndim) 
 j=i
 do while (j<=ndim) 
  !Add contribution due to harmonic component
  ! if (trim(verbos)=='high') print *, 'Jis(i,j,k): ',    (/(Jis(i,j,k),k=1,jval)/)
   if (trim(mode)=='harmonic') epot(i,j)=sum( (/(Kis(i,j,k)*harc(k),k=1,nb)/) )
   if (trim(mode)=='spline') epot(i,j)=is(splinc(1:jval-1,1:6),i,j,splinc(1,5),splinc(jval-1,6))
   if (trim(verbos)=='high') then
     print *, 'harmonic contrib:',i,j,sum( (/(Kis(i,j,k)*harc(k),k=1,nb)/) )
     print *, 'spline contrib:',i,j, is(splinc(1:jval-1,1:6),i,j,splinc(1,5),splinc(jval-1,6))
   end if ! verbos == high
  !Add contribution due to confining potential   
   if (i==j)  then
    epot(i,j)=epot(i,j)+cp*(Cfp(i,j)-1.d0)
    ekin(i,j)=(j**2+kpointvec(jkpoint)**2)*(w**2/(2*mass))
   else
    epot(i,j)=epot(i,j)+cp*Cfp(i,j)+(4*((-1)**(j+i)-1)*(b-a)*j*i)/((j**2-i**2)**2*dpi**2)*vpert
    ekin(i,j)=0.d0
   end if
   if (i/=j)  epot(j,i)=epot(i,j)
  j=j+1
 ! if (j>4) exit
 end do
 ndimarr(1:ndim,1:ndim)=epot(1:ndim,1:ndim)+ekin(1:ndim,1:ndim)

 if (trim(verbos)=='high') then
  ! print *, 'Cfp(6,All): ',    (/ ((Cfp(j,k),k=1,12),j=1,12)/)
  print *, 'epot ',i,' All ',epot(i,1:ndim)
  print *, 'ekin ',i,ekin(i,i)
 end if ! verbos == high
 ! exit
 i=i+1
end do

! Diagonalization of Hamiltonian

lwork=-1
! details for dsyev on netlib.org
! call DSYEV( JOBZ, UPLO, N, A, LDA, W, WORK, LWORK, INFO )
  call DSYEV('v','u',ndim,darr2,ndim,ndimvec(1:ndim),dvec(1:1),lwork,info )
  lwork=int(dvec(1))
if (trim(verbos)=='high')  print *, '1. lwork: ',lwork,' info: ',info
  if (ALLOCATED(work)==.false.) allocate (work(lwork))
  call DSYEV('v','u',ndim,ndimarr,ndim,ndimvec(1:ndim),work,lwork, info )

rvec(1:5)=ndimvec(1:5)*hartev
dvec(1:2)=(rvec(2)-rvec(1))*evj*(/ 1.d0/hplanck,1.d-2/(hplanck*clight) /) 
dvec(3:4)=(rvec(3)-rvec(2))*evj*(/ 1.d0/hplanck,1.d-2/(hplanck*clight) /) 

 ekpoints(jkpoint,1:mdim)=(ndimvec(1:mdim)+eref)*hartev
 jkpoint=jkpoint+1
end do ! Loop on k-points, only for band structure 
 

call DATE_AND_TIME(date, time)

! Output to screen
 write(*, "(' date: ',a8,2x,'time: ',a10)") trim(date),trim(time)

if ((trim(verbos)=='high').or.(trim(verbos)=='mid'))  then 
 write(*,"(' 2. lwork: ',F8.3,1x,' info: ',I4,1x,'lowest eigenvalues in eV: ',6F14.5,1x)") work(1:1),info,rvec(1:5)
 write(*,"(' resonance frequency in Hz (cm^-1): ',E18.8,1x,F18.8,1x,' 2nd res Hz (cm^-1): ',E18.8,1x,F18.8)") dvec(1:4)
 write(*,"(' zero point energy energy in cm^-1: ',F18.8,1x)") rvec(1)*evj*1.d-2/(hplanck*clight)
 write(*,"(' case harmonic oscillator proportions (1,3,5,7,9): ',6F14.5,1x)")rvec(1:5)/rvec(1)
end if 

if ((int(nkpoint).gt.1))  then ! Output and stop for band structure
 open(fg, file=outfile)
 write(fg,"('#kpt (pi/a), energy (eV);  pot[Ha]:',F18.8,1x,'date: ',a8,2x,'time: ',a10)")eref,trim(date),trim(time)
 write(my_fmt, '(a, i0, a)') '(1x,I4,', mdim+1, '(1x,F18.8))' ! scientific format: E18.11
 jkpoint=1 
 do while (jkpoint<=nkpoint)
  write(fg,my_fmt) jkpoint,kpointvec(jkpoint),ekpoints(jkpoint,1:mdim)
  jkpoint=jkpoint+1
 end do
 close(fg)
 deallocate (hhndimarr,hndimarr,ndimarr,ndimvec,work,epot,ekin)
 call exit(0)
end if



if (ndimvec(1).lt.0)  then 
 print *, 'WARNING: eigenstates with negative energies'
end if 

! Convert to Rydberg
ndimvec(:)=ndimvec(:)*2
eref=eref*2

! Output coefficients of eigenfunctions to file
 open(fg, file=vectorout)
 write(my_fmt, '(a, i0, a)') '(1x,I4,1x,E20.11,1x,E20.11,1x,E20.11,1x,', ndim, '(1x,E18.11))'
 write(fg,"('#inx, E[Ry],mean,sigma^2, coeffs Sin[i*Pi/2*(x + 1)], a[angst],b[angst]; E_ref[Ry]:',3F18.8,1x,'date: ',a8,2x,'time: ',a10)") eref,a,b,trim(date),trim(time)
 i=1
 do while (i<=ndim) ! setup matrices for first and second moment 
  j=i
  do while (j<=ndim)
   if (i.eq.j) then
    hndimarr(i,j)=0.0d0
    hhndimarr(i,j)=(2*j**2*dpi**2*(a**2+a*b+b**2)-3*(a-b)**2)/(6*j**2*dpi**2)
   else
    hndimarr(i,j)=(4*((-1)**(j+i)-1)*(b-a)*j*i)/((j**2-i**2)**2*dpi**2)
    hhndimarr(i,j)=(8*(a-b)*j*i*(a-b*(-1)**(j+i)))/((j**2-i**2)**2*dpi**2)
   end if
   hndimarr(j,i)=hndimarr(i,j)
   hhndimarr(j,i)=hhndimarr(i,j)
   j=j+1
  end do
  if ((trim(verbos)=='high').and.(i.lt.4)) then
   print *, 'hndimarr ',i,' All ',(/ (hndimarr(i,j) ,j=1,4)/)
   print *, 'hhndimarr ',i,' All ',(/ (hhndimarr(i,j),j=1,4)/)
  end if ! verbos == high
  i=i+1
 end do 
 hndimarr=MATMUL(hndimarr,ndimarr)
 hhndimarr=MATMUL(hhndimarr,ndimarr)
 i=1
 do while (i<ndim)
  meann=DOT_PRODUCT(ndimarr(:,i),hndimarr(:,i)) ! mean
  dbasa(1:1)=DOT_PRODUCT(ndimarr(:,i),hhndimarr(:,i))-meann**2 ! sigma**2
  write(fg,my_fmt) i,ndimvec(i)+eref,meann,dbasa(1:1),ndimarr(:,i)
 i=i+1
 end do
 close(fg)

! Output eigenvalues to file
 open(fg, file=outfile)
 i=1
 write(fg,"('#inx, x, pot[Ha]:',F18.8,1x,'date: ',a8,2x,'time: ',a10)") eref,trim(date),trim(time)
 write(fg,"('#v_res [Hz], [cm^-1]',1x,E18.8,1x,F18.5,1x,'v2_res [Hz], [cm^-1]',1x,E18.8,1x,F18.5,1x)") dvec(1:4)
 do while (i<=ndim)
  write(fg,"(1x,I4,1x,F18.10,1x)") i,(ndimvec(i)+eref) 
 i=i+1
 end do
 close(fg)  

! Calculate Ekin, Epot separately for lowest states
epot(1:ndim,1:ndim) = MATMUL(epot(1:ndim,1:ndim),ndimarr(1:ndim,1:ndim))
epot(1:ndim,1:ndim) = MATMUL(transpose(ndimarr(1:ndim,1:ndim)), epot(1:ndim,1:ndim))*hartev
ekin(1:ndim,1:ndim) = MATMUL(ekin(1:ndim,1:ndim),ndimarr(1:ndim,1:ndim))
ekin(1:ndim,1:ndim) = MATMUL(transpose(ndimarr(1:ndim,1:ndim)), ekin(1:ndim,1:ndim))  *hartev 
write(*,"(' Epot,Ekin, state 1 [eV]: ',F18.8,1x,F18.8,1x,F18.8,1x,F18.8,1x,F18.8,1x)") epot(1,1),ekin(1,1)
write(*,"(' Epot,Ekin, state 2 [eV]: ',F18.8,1x,F18.8,1x,F18.8,1x,F18.8,1x,F18.8,1x)") epot(2,2),ekin(2,2)
write(*,"(' Epot,Ekin, state 3 [eV]: ',F18.8,1x,F18.8,1x,F18.8,1x,F18.8,1x,F18.8,1x)") epot(3,3),ekin(3,3)
write(*,"(' Epot,Ekin, state 4 [eV]: ',F18.8,1x,F18.8,1x,F18.8,1x,F18.8,1x,F18.8,1x)") epot(4,4),ekin(4,4)
write(*,"(' Epot,Ekin, state 5 [eV]: ',F18.8,1x,F18.8,1x,F18.8,1x,F18.8,1x,F18.8,1x)") epot(5,5),ekin(5,5)

! Output fitted potential to file 
 open(fg, file=fifile)
 ndimarr(:,1)=(/(a+(b-a)/ndim*k,k=1,ndim-1)/)
 i=1
 write(fg,"('#inx  E_ref[Ry]:',F18.8,1x,'date: ',a8,2x,'time: ',a10)") eref,trim(date),trim(time)
 if (trim(mode)=='spline') then
  do while (i<ndim)
   ndimarr(i,2:2)=cp*ytoconfp(xtoy((/ ndimarr(i,1) /),a,b))
   ndimarr(i,3:3)=splin(ndimarr(i,1),splinc(1:jval-1,1:6)) 
   ndimarr(i,2)=sum(ndimarr(i,2:3))*2+eref ! conversion hartree to rydberg
   write(fg,"(1x,I4,1x,F18.10,1x,E18.10,1x)") i,ndimarr(i,1)*bohrangst,ndimarr(i,2) ! convert bohr to angst
  i=i+1
  end do
 else
  do while (i<ndim)
   ndimarr(i,2:2)=cp*ytoconfp(xtoy((/ ndimarr(i,1) /),a,b))
   ndimarr(i,3:3)=sum((/ ( sin(k*dpi/2 *(xtoy((/ ndimarr(i,1) /),a,b)+1.0D0))*harc(k),k=1,jval)/))
   ndimarr(i,2)=sum(ndimarr(i,2:3))*2+eref ! conversion hartree to rydberg
   write(fg,"(1x,I4,1x,F18.10,1x,E18.10,1x)") i,ndimarr(i,1)*bohrangst,ndimarr(i,2) ! convert bohr to angst
  i=i+1
  end do
 end if
 close(fg)


deallocate (hhndimarr,hndimarr,ndimarr,ndimvec,work,epot,ekin)


end program control_file
!amp j energ[Ry] mass[m_e]
!mass  123681.48839917
!0 0 -1189.44071304
!.62573784 1 -1189.45293046
