program control_file
  use eigenstate_utils
  use operations
 
  implicit none

  ! Input related variables
  character(len=100) :: buffer, puffer, label, outfile, vectorout,verbos
  integer :: pos, jval, ljval, ndim,i,j,k,ii,jj,kk,nkt
  integer, parameter :: fh = 15
  integer, parameter :: fg = 6
  integer, parameter :: ff = 7
  integer, parameter :: dm = 25 ! maximum lines that can be read
  integer, parameter :: ldm = 1*dm ! repartition of the integration interval
  integer     :: ios =0,ioss =0,line=0,lwork,info

  ! Control file variables
  real :: pill,  maxen
  real*8  , dimension(dm,dm) :: rarrn, rbrrn
  double precision :: mass, lbval, lcval, rbval, rcval, hh ,hg, br, bl,eref
  integer, dimension(dm) :: vector, ivec
  double precision , dimension(dm,3) :: ipd
  double precision , dimension(dm) :: rvec,dvec, dbasa, obasa, frq
  double precision , dimension(ldm) :: dlvec,dlved
  double precision , dimension(2,2) :: darr2, dbrr2
  double precision , dimension(3,3) :: darr3, dbrr3
  double precision , dimension(dm,dm) :: darrn, dbrrn, dcrrn, ddrrn
  double precision , dimension(:), allocatable :: ndimvec,work
  double precision , dimension(:,:), allocatable :: ndimarr

  open(fh, file='naliz.dat')

  ndim=5 !default value of number of basis functions
  outfile='eigenstates.dat'
  vectorout='eigenvecs.dat'
  verbos='low'

  ! ios is negative if an end of record condition is encountered or if
  ! an endfile condition was detected.  It is positive if an error was
  ! detected.  ios is zero otherwise.

  if (ios /= 0) then
    print *, 'Input naliz.dat does not exist'
    stop
  end if
  
  jval=1 ; ipd=0.0000
  do while (ios == 0)
     read(fh, '(A)', iostat=ios) puffer
     if (ios == 0) then
        line = line + 1

        ! Find the first instance of whitespace.  Split label and data.
        pos = scan(puffer, '    ')
        label = puffer(1:pos)
        buffer = puffer(pos+1:)
        

        select case (label)
        case ('verbos')
           read(buffer, *, iostat=ios) verbos
           print *, 'Read verbos: ', verbos
        case ('ndim')
           read(buffer, *, iostat=ios) ndim
           print *, 'Read ndim: ', ndim
        case ('vectorout')
           read(buffer, *, iostat=ios) vectorout
           print *, 'Read vectorout: ', vectorout
        case ('outfile')
           read(buffer, *, iostat=ios) outfile
           print *, 'Read outfile: ', outfile
        case ('maxen')
           read(buffer, *, iostat=ios) maxen
           print *, 'Read maxen: ', maxen
        case ('pill')
           read(buffer, *, iostat=ios) pill
           print *, 'Read pill: ', pill
        case ('mass')
           read(buffer, *, iostat=ios) mass
           print *, 'Read mass: ', mass
        case ('vector')
           read(buffer, *, iostat=ios) vector
           print *, 'Read vector: ', vector
        case default
           read(puffer, *, iostat=ioss) ipd(jval,:) 
           if  (abs(ipd(jval,3))>0) then
            write(*,"(1x,'ipd(',I4,'): ', F18.10,1x,I5,1x,F18.10)") jval,ipd(jval,1),int(ipd(jval,2)),ipd(jval,3)
            jval=jval+1
           else
            print *, 'Skipping invalid label at line', line
           end if
        end select
     end if
  end do
jval=jval-1

 allocate (ndimarr(ndim,ndim),ndimvec(ndim))
darr3=1
darr3(1,:)=(/ 1. , 2., 3. /)
darr3=RESHAPE( (/ 11, 21, 12, 31, 21, 12, 22, 13, 23 /), &
                   (/ 3, 3 /) )
!print *, 'darr3: ',darr3

print *, 'jval: ',jval
call hpsort(jval, ipd(1:jval,1), vector)
ipd(1:jval,2)=ipd(vector(1:jval),2)
rvec(1:jval)=ipd(1:jval,3)
call hpsort(jval,rvec(1:jval),ivec)
eref=ipd(ivec(1),3)
ipd(1:jval,3)=ipd(vector(1:jval),3)-eref

i=1
do while (i<=jval) 
 if (trim(verbos)=='high') write(*,"(1x,'ipd(',I4,'): ', F18.10,1x,I5,1x,F18.10)") i,ipd(i,1),int(ipd(i,2)),ipd(i,3)
 i=i+1
end do

!Find boundaries
nkt=3 ! number of points used at the boundary nd > nkt > 2 

darrn(1:nkt,1)=1
darrn(1:nkt,2)=ipd(1:nkt,1)
rvec(1:nkt)=reshape(1/ipd(1:nkt,3),(/nkt/))
dcrrn(1:2,1:2)=real(MATMUL(transpose(darrn(1:nkt,1:2)), darrn(1:nkt,1:2)))
dbrrn(1:2,1)=MATMUL(transpose(darrn(1:nkt,1:2)), rvec(1:nkt))
call dinverse(dcrrn(1:2,1:2))
darrn(1:2,1)=MATMUL(dcrrn(1:2,1:2),dbrrn(1:2,1))
lbval=darrn(1,1) ; lcval=darrn(2,1) ; bl=-lbval/lcval
if (trim(verbos)=='high') print *, 'lbval, lcval: ',darrn(1:2,1)
print *, 'left boundary at: ',bl

darrn(1:nkt,1)=1
darrn(1:nkt,2)=ipd(jval-nkt+1:jval,1)
rvec(1:nkt)=reshape(1/ipd(jval-nkt+1:jval,3),(/nkt/))
dcrrn(1:2,1:2)=real(MATMUL(transpose(darrn(1:nkt,1:2)), darrn(1:nkt,1:2)))
dbrrn(1:2,1)=MATMUL(transpose(darrn(1:nkt,1:2)), rvec(1:nkt))
call dinverse(dcrrn(1:2,1:2))
darrn(1:2,1)=MATMUL(dcrrn(1:2,1:2),dbrrn(1:2,1))
rbval=darrn(1,1) ; rcval=darrn(2,1); br=-rbval/rcval

if (trim(verbos)=='high') print *, 'rbval, rcval: ',darrn(1:2,1)
print *, 'right boundary at: ',br

!amplitudes mapped on 0,1
dbasa=ipd(1:jval,1)/(br-bl)+bl/(bl-br)
dbasa=reshape((/ 0.d0, dbasa(1:jval) , 1.d0 /),(/jval+2 /))
obasa=reshape((/ bl,  ipd(1:jval,1) , br /),(/jval+2 /))
if (trim(verbos)=='high') print *, 'dbasa: ', dbasa(1:jval)

!datan(1.d0)*4.d0 = pi
frq=(/ (i*datan(1.d0)*4.d0, i = 1, ndim) /)
if (trim(verbos)=='high') print *,'frq: ',frq(1:ndim)

ipd(1:jval,3)=1.d0
! Setup Hamiltonian

i=1
do while (i<=ndim) 
 j=3 !j=i
 do while (j<=ndim)  ! Generate points of integrand on dense mesh (by spine interpolation)
  ljval=jval+2
  rvec(1:ljval)=dsin(frq(i)*dbasa)*dsin(frq(j)*dbasa)*reshape((/ 0.0d0, ipd(1:jval,3) , 0.0d0 /),(/ljval /))*2/(br-bl)
    call spline(obasa(1:ljval),rvec(1:ljval),ljval,2/(br-bl)/lcval*1.d0*i*j*(datan(1.d0)*4.d0)**2,2/(br-bl)/rcval*(-1.d0)**(i+j+1)*i*j*(datan(1.d0)*4.d0)**2,dvec(1:ljval))
    call spline(obasa(1:ljval),rvec(1:ljval),ljval,0.d0,0.d0,dvec(1:ljval))

  ! 1.d0*i*j*(datan(1.d0)*4.d0)**2,(-1.d0)**(i+j-1)*i*j*(datan(1.d0)*4.d0)**2
  ! (rvec(2)-rvec(1))/(dbasa(2)-dbasa(1)),(rvec(ljval)-rvec(ljval-1))/(dbasa(ljval)-dbasa(ljval-1))
  k=1
  do while (k<=ldm) ! Generate points of integrand at all amplitudes
   dlved(k)=bl+dble((k-1.d0)/(ldm-1.d0))*(br-bl)
   call splint(obasa(1:ljval),rvec(1:ljval),dvec(1:ljval),ljval,dlved(k),dlvec(k))
   write(*,"(1x,'dlvec',I4,F18.10,F18.10)") k,dlved(k),dlvec(k)
   k=k+1
  end do
  ! call simpsn((br-bl)/(ldm-1),dlvec(1:ldm),ldm,ndimarr(i,j))
  ! call simpne(dlved(1:ldm),dlvec(1:ldm),ldm,ndimarr(i,j))
  call simpne(obasa(1:ljval),rvec(1:ljval),ljval,ndimarr(i,j))        
  if (i==j) ndimarr(i,j)=ndimarr(i,j)+j**2/(mass*2)
  if (trim(verbos)=='high') then
   if (i==j)  then
    print *, 'epot',i,j,ndimarr(i,j)
    print *, 'ekin',i,j,j**2/(mass*2) !(br-bl)*
   else
    print *, 'epot',i,j,ndimarr(i,j)
   end if ! (i=j)
  end if ! verbos == high
  if (i/=j)  ndimarr(j,i)=ndimarr(i,j)
  j=j+1
  exit ! if (j>3) exit
 end do
 exit
 i=i+1
end do
 
! Diagonalization of Hamiltonian

lwork=-1
! details for dsyev on netlib.org
! call DSYEV( JOBZ, UPLO, N, A, LDA, W, WORK, LWORK, INFO )
  call DSYEV('v','u',ndim,darr2,ndim,ndimvec(1:ndim),dvec(1:1),lwork,info )
  lwork=int(dvec(1))
if (trim(verbos)=='high')  print *, '1. lwork: ',lwork,' info: ',info
  allocate (work(lwork))
  call DSYEV('v','u',ndim,ndimarr,ndim,ndimvec(1:ndim),work,lwork, info )
if (trim(verbos)=='high')  print *, '2. lwork: ',dlvec(1:1),' info: ',info,'eigenvals: ',ndimvec(1:ndim)


! Output
 open(fg, file=outfile)
 i=1
 write(fg,"('#inx, e[Ry], eref[Ry]:',F18.8,1x)") eref
 dvec(1:2)=(ndimvec(2)-ndimvec(1))*(/ 3.28983880196602d15,109737.231563109d0 /)
 write(fg,"('#v_res [Hz], [cm^-1]',1x,E18.8,1x,F18.5,1x)") dvec(1),dvec(2)
 do while (i<=ndim)
  write(fg,"(1x,I4,1x,F18.8,1x)") i,ndimvec(i)
 i=i+1
 end do
 close(fg)
  
 

 deallocate (ndimarr,ndimvec,work)


end program control_file
!amp j energ[Ry] mass[m_e]
!mass  123681.48839917
!0 0 -1189.44071304
!.62573784 1 -1189.45293046
