program control_file
  use eigenstate_utils
  use operations
 
  implicit none

  ! Input related variables
  character(len=10000000) :: my_fmt
  character(len=100) :: buffer, puffer, label, outfile, vectorout,verbos ,fifile,date, time
  integer :: apos,pos, jval, ndim,i,j,k,jj,kk,nkt,np,nb   
  integer, parameter :: fh = 15
  integer, parameter :: fg = 6
  integer, parameter :: ff = 7
  integer, parameter :: dm = 50 ! maximum lines that can be read
  integer     :: ios =0,ioss =0,line=0,lwork,info
!  integer, dimension(3) :: today, now

  ! Control file variables
  real :: pill,  maxen
  real*8  , dimension(dm,dm) :: rarrn, rbrrn
  double precision, parameter :: hartev = 27.21138386 ! hartree to eV
  double precision, parameter :: evj =  1.60218d-19! electronvolt to joule
  double precision, parameter :: hplanck = 6.6260755d-34 ! SI
  double precision, parameter :: clight =  2.997924d8   ! SI
  double precision, parameter :: bohrm = 5.29177249d-11; !conversion from bohr to meter
  double precision, parameter :: bohrangst = bohrm*1d10 !conversion from bohr to angst
  double precision :: mass, cp,hh ,hg, a, b,eref,ci,si ,ymax
  integer, dimension(dm) :: vector, ivec
  double precision , dimension(dm,3) :: ipd
  double precision , dimension(dm) :: rvec,dvec, dbasa, frq, harc=0.d0
  double precision , dimension(2,2) :: darr2, dbrr2
  double precision , dimension(3,3) :: darr3, dbrr3
  double precision , dimension(dm,dm) :: bas, darrn, ddrrn
  double precision , dimension(:), allocatable :: ndimvec,work
  double precision , dimension(:,:), allocatable :: ndimarr,epot,ekin

  open(fh, file='naliz.dat')

  ndim=100 !default value of number of basis functions
  outfile='eigenstates.dat'
  vectorout='eigenvecs.dat'
  verbos='low'
  fifile='fit.dat'
  maxen=10.0d10

  ! ios is negative if an end of record condition is encountered or if
  ! an endfile condition was detected.  It is positive if an error was
  ! detected.  ios is zero otherwise.

  if (ios /= 0) then
    print *, 'Input naliz.dat does not exist'
    stop
  end if
  
  jval=1 ; ipd=0.0000 ; ymax=0.d0
  do while (ios == 0)
     read(fh, '(A)', iostat=ios) puffer
     if (ios == 0) then
        line = line + 1
        ! Find the first instance of whitespace.  Split label and data.
        apos = scan(puffer,"abcdefghijklmnopqrstuvwxyz-123456789")
        pos = scan(puffer(apos:), ' ')
        label = puffer(apos:pos+apos-1)
        buffer = puffer(apos+pos:)
   !       write(*,*) scan(puffer,"abcdefghijklmnopqrstuvwxyz-123456789") ,label
        select case (label)
        case ('verbos')
           read(buffer, *, iostat=ios) verbos ! mid high low
           print *, 'Read verbos: ', verbos
        case ('ndim')
           read(buffer, *, iostat=ios) ndim
           print *, 'Read ndim: ', ndim
        case ('vectorout')
           read(buffer, *, iostat=ios) vectorout
           print *, 'Read vectorout: ', vectorout
        case ('outfile')
           read(buffer, *, iostat=ios) outfile
           print *, 'Read outfile: ', outfile
        case ('maxen')
           read(buffer, *, iostat=ios) maxen ! in rydberg
           print *, 'Read maxen: ', maxen
        case ('pill')
           read(buffer, *, iostat=ios) pill
           print *, 'Read pill: ', pill
        case ('ymax')
           read(buffer, *, iostat=ios) ymax ! in bohr
           print *, 'Read ymax: ', ymax,' (',ymax/bohrangst,' bohr)' 
           ymax=ymax/bohrangst
        case ('mass')
           read(buffer, *, iostat=ios) mass ! in units of electron mass
           print *, 'Read mass: ', mass
        case ('vector')
           read(buffer, *, iostat=ios) vector
           print *, 'Read vector: ', vector
        case default
           read(puffer, *, iostat=ioss) ipd(jval,:) 
           if  (scan(label,'aeioud#').eq.0) then ! test if object is a number
            write(*,"(1x,'ipd(',I4,'): ', F18.10,1x,I5,1x,F18.10)") jval,ipd(jval,1),int(ipd(jval,2)),ipd(jval,3)
            jval=jval+1
           else
            print *, 'Skipping invalid label at line', line
           end if
        end select
     end if
  end do
jval=jval-1

allocate (ndimarr(ndim,ndim),epot(ndim,ndim),ekin(ndim,ndim),ndimvec(ndim))
darr3=1

print *, 'number datapoints read (jval): ',jval

! convert angstroms to bohr and ryd to hartree
ipd(1:jval,1)=ipd(1:jval,1)/bohrangst
ipd(1:jval,3)=ipd(1:jval,3)/2.d0


call hpsort(jval, ipd(1:jval,1), vector)
ipd(1:jval,2)=ipd(vector(1:jval),2)
rvec(1:jval)=ipd(1:jval,3)
call hpsort(jval,rvec(1:jval),ivec)
eref=ipd(ivec(1),3)
ipd(1:jval,3)=ipd(vector(1:jval),3)-eref

i=1   ! drop energies which are too high 
do while (i<=jval) 
 if (trim(verbos)=='high') write(*,"(1x,'ipd(',I4,'): ', F18.10,1x,I5,1x,F18.10)") i,ipd(i,1),int(ipd(i,2)),ipd(i,3)
 if (ipd(i,3).gt.maxen/2.d0) then
  ipd(i:jval-1,:)=ipd(i+1:jval,:)
  jval=jval-1
 else
  i=i+1
 end if
end do
print *, 'number datapoints used: ',jval



!Find boundaries
nkt=2 ! number of points used at the boundary nd > nkt > 2 

bas(1:nkt,1)=1.d0
bas(1:nkt,2)=1.d0/ipd(1:nkt,1)
bas(1:2,1)=linearfit(reshape(1.d0/ipd(1:nkt,3),(/nkt/)),bas(1:nkt,1:2))
a=-bas(2,1)/bas(1,1)
if ((a.lt.1.5*ipd(1,1)).or.(a/ipd(1,1).le.0)) then
  print *, 'WARNING: lower boundary found at: ',a*bohrangst
  a=1.2*ipd(1,1)
  print *, 'WARNING: lower boundary reset to: ',a*bohrangst
else if (trim(verbos)=='high') then
  print *,' lower boundary at: ',a
end if

bas(1:nkt,1)=1.d0
bas(1:nkt,2)=1/ipd(jval-nkt+1:jval,1)
bas(1:2,1)=linearfit(reshape(1.d0/ipd(jval-nkt+1:jval,3),(/nkt/)),bas(1:nkt,1:2))
b=-bas(2,1)/bas(1,1)
if ((b.gt.1.5*ipd(jval,1)).or.(b/ipd(jval,1).le.0)) then
  print *, 'WARNING: upper boundary found at: ',b*bohrangst
  b=1.2*ipd(jval,1)
  print *, 'WARNING: upper boundary reset to: ',b*bohrangst
else if (trim(verbos)=='high') then
  print *,' upper boundary at: ',b*bohrangst
end if

if (abs(ymax).gt.10d-8) then
  b=abs(ymax)
  a=-b
  print *, 'WARNING: boundary provided: ',a*bohrangst,' and ',b*bohrangst,' angst'
end if

!if (trim(verbos)=='high') print *, 'upper boundary at: ',b

!amplitudes mapped on 0,1
dbasa=xtoy(ipd(1:jval,1),a,b)
if (trim(verbos)=='high') print *, 'dbasa: ', dbasa(1:jval)

!amplitude of confining potential 1/(1-y^2)
darrn(1:2*nkt,1)=reshape((/ dbasa(1:nkt),dbasa(jval-nkt+1:jval) /),(/2*nkt/))
darrn(1:2*nkt,2)=reshape((/ ipd(1:nkt,3),ipd(jval-nkt+1:jval,3) /),(/2*nkt/))
cp=dot_product(darrn(1:2*nkt,2),darrn(1:2*nkt,2))/dot_product(darrn(1:2*nkt,2),ytoconfp(darrn(1:2*nkt,1)))
if (trim(verbos)=='high') print *, '1/(1-y^2)-1 contribution: ',cp

!frq=(/ (i*dpi, i = 1, ndim) /)
!if (trim(verbos)=='high') print *,'frq: ',frq(1:ndim)

!decomposition into monoms harmonic functions Sin[ j Pi/2 (x + 1)] 
rvec(1:jval)=ipd(1:jval,3)-cp*ytoconfp(dbasa(1:jval))

if (trim(verbos)=='high') print *, 'rvec: ',rvec(1:jval)

np=jval ! number of points in the fit
if (np.gt.jval ) rvec(jval+1:np)=0
i=1 
nb=jval-2 ! number basis vectors
do while (i<=nb)
 ! bas(1:np,i)=reshape((/ dbasa(1:jval),xtoy((/ a+(ipd(1,1)-a)/2, b+(ipd(jval,1)-b)/2 /),a,b) /),(/ np /))**i
 ! bas(1:np,i)=reshape((/ dbasa(1:jval),xtoy((/ a+(ipd(1,1)-a)/2, b+(ipd(jval,1)-b)/2 /),a,b) /),(/ np /))
 bas(1:np,i)= dbasa(1:jval)
 bas(1:np,i)=sin(i*dpi/2 *(bas(1:np,i)+1.0D0))
 i=i+1
end do
if (trim(verbos)=='high') print *, 'size(bas) : ', size(bas)
if (trim(verbos)=='high') print *, 'bas(1:np,1) : ',bas(1:np,1)
if (trim(verbos)=='high') print *, 'bas(1:np,2) : ',bas(1:np,2)

harc(1:jval)=linearfit(rvec(1:np),bas(1:np,1:nb))
if (trim(verbos)=='high') print *, 'harmonic components: ',harc(1:nb)

! Check Romberg Algorythem

if (trim(verbos)=='high') then
 !call rombe (func,b,a,aserr,relerr,error,result,ind)
 !call rombe (func,b,a,1.d-15,1.d-15,darrn(1,3),darrn(1,4),i)
 !print *, 'error: ', darrn(1,3)
 !print *, 'int_a^b y^2 dy: ', darrn(1,4)
 !print *, 'ind: ', i
 i=1
 do while (i<=6) 
  call cisi(dpi*i,ci,si)
  rvec(i)=ci
  i=i+1
 end do
 print *, 'CosIntegral[{1,2,3,4,5,6}* Pi] ', rvec(1:6)
 print *, 'CI[4 Pi],-CI[6], Log[3/2] ', rvec(4),rvec(6),dlog((1+5)/dble(iabs(1-5)))
 print *, 'CI[4 Pi],-CI[6], Log[3/2] ', rvec(4),rvec(6),dlog((5+1)/dble(iabs(5-1)))

end if

! Setup Hamiltonian

i=1
do while (i<=ndim) 
 j=i
 do while (j<=ndim) 
  !Add contribution due to monoms
  ! if (trim(verbos)=='high') print *, 'Jis(i,j,k): ',    (/(Jis(i,j,k),k=1,jval)/)
    epot(i,j)=sum( (/(Kis(i,j,k)*harc(k),k=1,nb)/) )
   if (trim(verbos)=='high') then
     print *, 'monom contrib:',i,j,epot(i,j)
   end if ! verbos == high
  !Add contribution due to confining potential   
   if (i==j)  then
    epot(i,j)=epot(i,j)+cp*(Cfp(i,j)-1.d0)
    ekin(i,j)=j**2/(8*mass)*dpi**2
   else
    epot(i,j)=epot(i,j)+cp*Cfp(i,j)
    ekin(i,j)=0.d0
   end if
   ndimarr(i,j)=(epot(i,j)+ekin(i,j))/(b-a)
   if (i/=j)  ndimarr(j,i)=ndimarr(i,j)
  j=j+1
 ! if (j>4) exit
 end do
 if (trim(verbos)=='high') then
  ! print *, 'Cfp(6,All): ',    (/ ((Cfp(j,k),k=1,12),j=1,12)/)
  print *, 'epot ',i,' All ',epot(i,1:ndim)
  print *, 'ekin ',i,ekin(i,i)
 end if ! verbos == high
 ! exit
 i=i+1
end do

! factor

epot=epot*(b-a)/2.0D0
 
! Diagonalization of Hamiltonian

lwork=-1
! details for dsyev on netlib.org
! call DSYEV( JOBZ, UPLO, N, A, LDA, W, WORK, LWORK, INFO )
  call DSYEV('v','u',ndim,darr2,ndim,ndimvec(1:ndim),dvec(1:1),lwork,info )
  lwork=int(dvec(1))
if (trim(verbos)=='high')  print *, '1. lwork: ',lwork,' info: ',info
  allocate (work(lwork))
  call DSYEV('v','u',ndim,ndimarr,ndim,ndimvec(1:ndim),work,lwork, info )

rvec(1:5)=ndimvec(1:5)*hartev
dvec(1:2)=(rvec(2)-rvec(1))*evj*(/ 1.d0/hplanck,1.d-2/(hplanck*clight) /) 

 call DATE_AND_TIME(date, time)


! Output to screen
 write(*, "(' date: ',a8,2x,'time: ',a10)") trim(date),trim(time)

if ((trim(verbos)=='high').or.(trim(verbos)=='mid'))  then 
 write(*,"(' 2. lwork: ',F8.3,1x,' info: ',I4,1x,'lowest eigenvalues in eV: ',6F14.5,1x)") work(1:1),info,rvec(1:5)
 write(*,"(' resonance frequency in Hz (cm^-1): ',E18.8,1x,F18.8,1x)") dvec(1),dvec(2)
 write(*,"(' zero point energy energy in cm^-1: ',F18.8,1x)") rvec(1)*evj*1.d-2/(hplanck*clight)
 write(*,"(' case harmonic oscillator proportions (1,3,5,7,9): ',6F14.5,1x)")rvec(1:5)/rvec(1)
end if 

if (ndimvec(1).lt.0)  then 
 print *, 'WARNING: eigenstates with negative energies'
end if 

! Convert to Rydberg
ndimvec(:)=ndimvec(:)*2
eref=eref*2

! Output coefficients of eigenfunctions to file
 open(fg, file=vectorout)
 write(my_fmt, '(a, i0, a)') '(1x,I4,1x,E20.11,1x,', ndim, '(1x,E18.11))'
 i=1
 write(fg,"('#inx, E[Ry], coeffs Sin[i*Pi/2*(x + 1)], a[angst],b[angst]; E_ref[Ry]:',3F18.8,1x,'date: ',a8,2x,'time: ',a10)") eref,a,b,trim(date),trim(time)
 do while (i<ndim)
  write(fg,my_fmt) i,ndimvec(i)+eref,ndimarr(:,i)
 i=i+1
 end do
 close(fg)

! Output eigenvalues to file
 open(fg, file=outfile)
 i=1
 write(fg,"('#inx, x, pot[Ha]:',F18.8,1x,'date: ',a8,2x,'time: ',a10)") eref,trim(date),trim(time)
 write(fg,"('#v_res [Hz], [cm^-1]',1x,E18.8,1x,F18.5,1x)") dvec(1),dvec(2)
 do while (i<=ndim)
  write(fg,"(1x,I4,1x,F18.10,1x)") i,(ndimvec(i)+eref) 
 i=i+1
 end do
 close(fg)  

! Output fitted potential to file 
 open(fg, file=fifile)
 ndimarr(:,1)=(/(a+(b-a)/ndim*k,k=1,ndim-1)/)
 i=1
 write(fg,"('#inx  E_ref[Ry]:',F18.8,1x,'date: ',a8,2x,'time: ',a10)") eref,trim(date),trim(time)
 do while (i<ndim)
  ndimarr(i,2:2)=cp*ytoconfp(xtoy((/ ndimarr(i,1) /),a,b))
  ndimarr(i,3:3)=sum((/ ( sin(k*dpi/2 *(xtoy((/ ndimarr(i,1) /),a,b)+1.0D0))*harc(k),k=1,jval)/))
  ndimarr(i,2)=sum(ndimarr(i,2:3))*2+eref ! conversion hartree to rydberg
  write(fg,"(1x,I4,1x,F18.10,1x,E18.10,1x)") i,ndimarr(i,1)*bohrangst,ndimarr(i,2) ! convert bohr to angst
 i=i+1
 end do
 close(fg)

deallocate (ndimarr,ndimvec,work,epot,ekin)


end program control_file
!amp j energ[Ry] mass[m_e]
!mass  123681.48839917
!0 0 -1189.44071304
!.62573784 1 -1189.45293046
