module electrostatic

use operations

implicit none

contains

!subroutine quad(p,charges,basis,q)

!real*8, intent(in)     :: p(3),charges(:,:),basis(3,3)
!real*8, intent(out)    :: q(6)
!real*8                 :: t(3),rq,theq
!integer                :: n,i
!
!n=size(charges(:,1))
!
!print*,n
!
!q=0
!
!do i=1,n
!theq=charges(i,4)
!call vettore(p,charges(i,1:3),t(1),t(2),t(3),basis)
!rq=norm(t)
!rq=rq*rq
!q(1)=q(1)+(3*t(1)*t(1)-rq)*theq
!q(2)=q(2)+(3*t(2)*t(2)-rq)*theq
!q(3)=q(3)+(3*t(3)*t(3)-rq)*theq
!q(4)=q(4)+theq*3.d0*t(1)*t(2)
!q(5)=q(5)+theq*3.d0*t(1)*t(3)
!q(6)=q(6)+theq*3.d0*t(3)*t(2)
!enddo
!
!return
!
!end subroutine quad

subroutine quadrupole(rho,quad)

real*8, intent(in)       :: rho(:,:)
real*8, intent(out)      :: quad(3,3)
real*8                   :: vr(3),nvr,q
integer                  :: i 

quad=0.d0
do i=1,size(rho(:,1))

   vr=rho(i,1:3)
   nvr=norm(vr)
   q=rho(i,4)
   quad(1,1)=quad(1,1)+q*(3.d0*vr(1)*vr(1)-nvr*nvr)
   quad(2,2)=quad(2,2)+q*(3.d0*vr(2)*vr(2)-nvr*nvr)
   quad(3,3)=quad(3,3)+q*(3.d0*vr(3)*vr(3)-nvr*nvr)
   quad(1,2)=quad(1,2)+q*(3.d0*vr(1)*vr(2))
   quad(1,3)=quad(1,3)+q*(3.d0*vr(1)*vr(3))
   quad(2,1)=quad(1,2)
   quad(2,3)=quad(2,3)+q*(3.d0*vr(2)*vr(3))
   quad(3,1)=quad(1,3)
   quad(3,2)=quad(2,3)

enddo

end subroutine quadrupole

subroutine dipfield(r,d,df)

real*8, intent(in)   :: r(3) , d(3)
real*8, intent(out)  :: df(3) 
real*8               :: norm_r,fac1,rv(3),s

fac1=1.d0
norm_r=norm(r)
rv=r/norm_r
s=scalar(d,rv)
df=1.d0/fac1*(3.d0*s*rv - d)/(norm_r*norm_r*norm_r)

end subroutine dipfield

real*8 function dippot(r,d)

real*8, intent(in)   :: r(3) , d(3)
real*8               :: n3,eps

eps=1.d-06
n3=norm(r)
dippot=0.d0
if(n3.ge.eps)then
 n3=n3*n3*n3
 dippot=scalar(r,d)/n3
endif

end function dippot

real*8 function quadpot(r,q)
real*8, intent(in)   :: r(3) , q(3,3)
real*8               :: n3,eps,n(3)
integer              :: i,j

eps=1.d-06
n3=norm(r)
quadpot=0.d0

if(n3.ge.eps)then

 n=r/n3
 n3=n3*n3*n3
 do i=1,3
  do j=1,3
! quadpot=quadpot+q(i,j)*n(i)*n(j)
!  if(i.eq.j)then
!   quadpot=quadpot+q(i,i)*(3.d0*n(i)*n(i)-1.d0)
!  else
!   quadpot=quadpot+q(i,j)*3.d0*n(i)*n(j)
    quadpot=quadpot+q(i,j)*n(i)*n(j)
!  endif
  enddo
 enddo
!quadpot=quadpot/2.d0/n3
 quadpot=quadpot/2.d0/n3

endif


end function quadpot

real*8 function qpot(r,q)

real*8, intent(in)   :: r(3) , q
real*8               :: nr,eps

eps=1.d-06
qpot=0.d0
nr=norm(r)
if(nr.ge.eps) then
qpot=q/nr
endif

end function qpot

subroutine qfield(r,q,qf)

real*8, intent(in)   :: r(3) , q
real*8, intent(out)  :: qf(3) 
real*8               :: norm_r,fac1,rv(3)

fac1=1.d0
norm_r=norm(r)
rv=r/norm_r
qf=1/fac1*(q*rv)/(norm_r*norm_r)


end subroutine qfield

subroutine multipole(rho,l,q)

real*8, intent(in)       :: rho(:,:)
integer, intent(in)      :: l
complex*16, intent(out)  :: q(-l:l)
real*8                   :: s(3)
integer                  :: i,n,m

n=size(rho(:,1))
q=cmplx(0.d0,0.d0)
do i=1,n
 call cart2spher(rho(i,1:3),s)
!print*,rho(i,1:3),s(1:3)
 do m=-l,l
  if(l.eq.0) then
  q(m)=q(m)+rho(i,4)*conjg( ylm(l,m,s(2),s(3)) )*(1.d0) *dsqrt(16.d0*datan(1.d0)/(2.d0*l+1))
  else
  q(m)=q(m)+rho(i,4)*conjg( ylm(l,m,s(2),s(3)) )*(s(1)**(l)) *dsqrt(16.d0*datan(1.d0)/(2.d0*l+1))
  endif
! print*,rho(i,4)
! print*,ylm(l,m,s(2),s(3))
 enddo
enddo


end subroutine multipole

subroutine rmultipole(rho,l,q)

real*8, intent(in)       :: rho(:,:)
integer, intent(in)      :: l
real*8, intent(out)  :: q(-l:l)
real*8                   :: s(3)
integer                  :: i,n,m

n=size(rho(:,1))
q=0.d0
do i=1,n
 call cart2spher(rho(i,1:3),s)
!print*,rho(i,1:3),s(1:3)
 do m=-l,l
  if(l.eq.0) then
  q(m)=q(m)+rho(i,4)*( rylm(l,m,s(2),s(3)) )*(1.d0) *dsqrt(16.d0*datan(1.d0)/(2.d0*l+1))
  else
  q(m)=q(m)+rho(i,4)*( rylm(l,m,s(2),s(3)) )*(s(1)**(l)) *dsqrt(16.d0*datan(1.d0)/(2.d0*l+1))
  endif
! print*,rho(i,4)
! print*,ylm(l,m,s(2),s(3))
 enddo
enddo


end subroutine rmultipole
!subroutine m_field(r,l,q,f)

!complex*16, intent(in)  :: q(-l:l)
!integer, intent(in)     :: l
!real*8, intent(in)      :: r(3)
!real*8, intent(out)     :: f(3)
!real*8                  :: s(3)
!complex*16              :: dum(3),y_lm(-l:l)
!integer                 :: m
!
!pi=4.d0*datan(1.d0)
!
!dum=cmplx(0.d0,0.d0)
!call cart2spher(r,s)
!do m=-l,l
!y_lm(m)=ylm(l,m,s(2),s(3)) * (1.d0/(2*l+1))
!enddo
!
!do m=-l,-l
! if(m.eq.0) then
!   dum(1)=dum(1)-q(m)*y_lm(m)
!   dum(2)=dum(2)+q(m)*  /s(1)
!   dum(3)=dum(3)+q(m)*  /s(1)/dsin(s(2))
! else if(m.eq.l)then 
!   dum(1)=dum(1)-q(m)*y_lm(m)
!   dum(2)=dum(2)+q(m)*
!   dum(3)=dum(3)+q(m)*
! else 
!   dum(1)=dum(1)-q(m)*y_lm(m)
!   dum(2)=dum(2)+q(m)*
!   dum(3)=dum(3)+q(m)*
! end select
!enddo
!
!dum=dum/(s(1)**(l+2))
!
!mpot=real(dum)*4.d0*pi
!
!
!end subroutine m_field

real*8 function mpot(r,l,q)

integer, intent(in)     :: l
complex*16, intent(in)  :: q(-l:l)
real*8, intent(in)      :: r(3)
real*8                  :: s(3)
complex*16              :: dum
integer                 :: m

pi=4.d0*datan(1.d0)

dum=cmplx(0.d0,0.d0)
call cart2spher(r,s)

do m=-l,l
 dum=dum+q(m)*dsqrt(4.d0*pi/(2.d0*l+1.d0))*ylm(l,m,s(2),s(3))/(s(1)**(l+1))
enddo
mpot=real(dum)  !*4.d0*pi


end function mpot


end module electrostatic
