module images

use operations
 
implicit none

type bmp
 character(len=1), pointer        :: rgb(:,:,:)
 integer                          :: nx,ny
end type bmp
 
contains

subroutine uniform2bmp(c,b,iunit)
type(bmp), intent(out)      :: b
real*8, intent(inout)       :: c(:,:)
integer, intent(in)         :: iunit
integer                     :: i,j,h,v,maxcolor,npalette
integer,allocatable         :: gray(:,:,:),colormap(:,:)
real*8                      :: dz
real*8                      :: maxz,minz


h=size(c,1)
v=size(c,2)
maxz=maxval(c)
minz=minval(c)
read(iunit,*),npalette
allocate(colormap(0:npalette-1,3))
do i=0,npalette-1
 read(iunit,*)colormap(i,1:3)
enddo
maxcolor=maxval(colormap(0:npalette-1,1:3))
do i=0,npalette-1
 do j=1,3
  colormap(i,j)=nint(colormap(i,j)*1.d0/(maxcolor*1.d0)*(npalette-1))
 enddo
! write(18,*)colormap(i,1:3)
enddo
dz=maxz-minz
!print*,dz
allocate(gray(h,v,3))
gray=0.d0
do i=1,h
 do j=1,v
    gray(i,j,1:3)=colormap(nint(  (c(i,j)-minz)/dz*(npalette-1)  ),1:3)
 enddo
enddo

call make_bmp(b,gray)

deallocate(gray)
deallocate(colormap)


end subroutine uniform2bmp

!subroutine real2bmp(c,b,ph,iunit)
!type(bmp), intent(out)      :: b
!real*8, intent(inout)       :: c(:,:,:)
!integer, intent(in)         :: ph,iunit
!integer                     :: i,j,h,v,pv,thei,thej,vi(3)
!integer,allocatable         :: gray(:,:)
!real*8                      :: dx(3,3),dh,dv,dz,x,y,z,ddh,ddv
!real*8                      :: maxx,maxy,maxz,minx,miny,minz,vr(3)
!real*8, allocatable         :: r(:,:,:)

!h=size(c,2)
!v=size(c,3)

!maxx=maxval(c(1,:,:))
!maxy=maxval(c(2,:,:))
!maxz=maxval(c(3,:,:))
!minx=minval(c(1,:,:))
!miny=minval(c(2,:,:))
!minz=minval(c(3,:,:))

!vr=0.d0
!dh=maxx-minx
!dv=maxy-miny
!dz=maxz-minz
!pv=nint(ph*(dv/dh))
!ddh=dh/(ph-1)
!ddv=dv/(pv-1)
!allocate(gray(ph,pv))
!gray=0.d0

!do i=1,h
! do j=1,v
!  c(1:3,i,j)=c(1:3,i,j)-c(1:3,1,1) 
! enddo
!enddo

!dx=0.d0
!dx(1,1:2)=c(1:2,h,1)/(h-2)
!dx(2,1:2)=c(1:2,1,v)/(v-2)
!dx(3,3)=dz/255.d0

!do i=1,ph-1
 
! vr(1)=ddh*(i-1)
! do j=1,pv-1
!    vr(2)=ddv*(j-1)
!    call ijk(vi,vr,dx)
!    gray(i,j)=nint(c(3,vi(1),vi(2))/dz*255)
! enddo
!enddo
!gray(ph,1:pv)=gray(1,1:pv)
!gray(1:ph-1,pv)=gray(1:ph-1,1)

!call make_bmp(b,gray)

!deallocate(gray)

!end subroutine real2bmp


       subroutine write_bmp(b,iunit)
! local
       type(bmp), intent(in) :: b
       integer, intent(in)   :: iunit
       character(len=12)     :: fnameout
       integer               :: i, j, k
       integer               :: itmp, icnt,ihpixf,jvpixf
       character(len=14)     :: frmtstr
       character(len=54)     :: headmsw
       character(len=4 )     :: byt4
       character(len=2 )     :: byt2
! choices
       integer               :: ipixout
       ihpixf=b%nx
       jvpixf=b%ny


! BMP (24bit depth)... this part works only when width is multiple of 4.


! header 1 (file header ; 1--14 byte)
         headmsw( 1: 2) = 'BM'             ! declaring this is BMP file
         itmp = 54 + ihpixf * jvpixf * 3 ! total file size = header + data
         call num2bit4(itmp,byt4)
         headmsw( 3: 6) = byt4(1:4)
         itmp = 0                        ! may be 0
         call num2bit2(itmp,byt2)
         headmsw( 7: 8) = byt2(1:2)
         itmp = 0                        ! may be 0
         call num2bit2(itmp,byt2)
         headmsw( 9:10) = byt2(1:2)
         itmp = 54                       ! must be 54 : total length of header
         call num2bit4(itmp,byt4)
         headmsw(11:14) = byt4(1:4)
! header 2 (bit-map header ; 13--54 byte)
         itmp = 40                       ! must be 40 : length of bit-map header
         call num2bit4(itmp,byt4)
         headmsw(15:18) = byt4(1:4)
         itmp = ihpixf                   ! width
         call num2bit4(itmp,byt4)
         headmsw(19:22) = byt4(1:4)
         itmp = jvpixf                   ! height
         call num2bit4(itmp,byt4)
         headmsw(23:26) = byt4(1:4)
         itmp = 1                        ! must be 1
         call num2bit2(itmp,byt2)
         headmsw(27:28) = byt2(1:2)
         itmp = 24                       ! must be 24 : color depth in bit.
         call num2bit2(itmp,byt2)
         headmsw(29:30) = byt2(1:2)
         itmp = 0                        ! may be 0 : compression method index
         call num2bit4(itmp,byt4)
         headmsw(31:34) = byt4(1:4)
         itmp = 0                        ! may be 0 : file size if compressed
         call num2bit4(itmp,byt4)
         headmsw(35:38) = byt4(1:4)
         itmp = 0                        ! arbit. : pixel per meter, horizontal
         call num2bit4(itmp,byt4)
         headmsw(39:42) = byt4(1:4)
         itmp = 0                        ! arbit. : pixel per meter, vertical
         call num2bit4(itmp,byt4)
         headmsw(43:46) = byt4(1:4)
         itmp = 0                        ! may be 0 here : num. of color used
         call num2bit4(itmp,byt4)
         headmsw(47:50) = byt4(1:4)
         itmp = 0                        ! may be 0 here : num. of important color
         call num2bit4(itmp,byt4)
         headmsw(51:54) = byt4(1:4)

! writing header part
         write(iunit,'(a54,$)') headmsw(1:54)
! image data
         itmp = ihpixf * jvpixf * 3
         write(frmtstr,'(''('',i8.8,''A,$)'')') itmp
         write(iunit,fmt=frmtstr) &
          (((b%rgb(k,i,j),k=3,1,-1),i=1,ihpixf),j=1,jvpixf) ! writing in BGR order, not RGB.


       end subroutine write_bmp

! --------------------------------------
! convert number to 8-bit characters
! --------------------------------------

       subroutine num2bit4(inum,byt4)
       
       integer,intent(in)            ::  inum
       character(len=4), intent(out) :: byt4
       integer                       :: itmp1, itmp2
       itmp1 = inum
       itmp2 = itmp1 / 256**3
       byt4(4:4) = char(itmp2)
       itmp1 =-itmp2 * 256**3 +itmp1
       itmp2 = itmp1 / 256**2
       byt4(3:3) = char(itmp2)
       itmp1 =-itmp2 * 256**2 +itmp1
       itmp2 = itmp1 / 256
       byt4(2:2) = char(itmp2)
       itmp1 =-itmp2 * 256    +itmp1
       byt4(1:1) = char(itmp1)

       end subroutine num2bit4

! ------

       subroutine num2bit2(inum,byt2)

       integer,intent(in)            ::  inum
       character(len=2), intent(out) :: byt2
       integer                       :: itmp1, itmp2
       itmp1 = inum
       itmp2 = itmp1 / 256
       byt2(2:2) = char(itmp2)
       itmp1 =-itmp2 * 256 + itmp1
       byt2(1:1) = char(itmp1)

       end subroutine num2bit2


       subroutine    create_bmp(b,m,n)
        type(bmp), intent(out)      :: b
        integer, intent(in)         :: m,n
        integer                     :: itmp

        allocate(b%rgb(3,m,n))
         itmp = mod(m, 4)
         if (itmp .NE. 0) then
           write(*,*) 'width must be multiple of 4'
           stop
         endif
        b%nx=m
        b%ny=n
       end subroutine create_bmp

       subroutine make_bmp(b,c)
       type(bmp), intent(out)       :: b
       integer, intent(in)          :: c(:,:,:)
       integer                      :: i,j,m,n
     
       n=size(c(1,:,1))
       m=size(c(:,1,1))
       call create_bmp(b,m,n)
       do i=1,m
        do j=1,n
           b%rgb(1,i,j)=char(c(i,j,1))
           b%rgb(2,i,j)=char(c(i,j,2))
           b%rgb(3,i,j)=char(c(i,j,3))
        enddo
       enddo

       end subroutine make_bmp

       subroutine free_bmp(b)
       type(bmp), intent(inout)   :: b
       deallocate(b%rgb)
       end subroutine free_bmp

end module images
