module operations

use geometry
use cg

IMPLICIT NONE
 
real*8, save :: pi

contains

logical function crosslines(l1,l2)

real*8, intent(in)      :: l1(:,:),l2(:,:)
real*8, allocatable     :: p1(:),p2(:),p3(:),p4(:),pc(:)
integer                 :: n1,n2,i,j
logical                 :: flag

crosslines=.true.

n1=size(l1(:,1))
n2=size(l2(:,1))
allocate(p1(size(l1(1,:))))
allocate(p2(size(l1(1,:))))
allocate(p3(size(l1(1,:))))
allocate(p4(size(l1(1,:))))
allocate(pc(size(l1(1,:))))

do i=1,n1-1
 p1=l1(i,1:2)
 p2=l1(i+1,1:2)
 do j=1,n2-1
  p3=l2(j,1:2)
  p4=l2(j+1,1:2)
  call crosspoint(p1,p2,p3,p4,pc,flag)
  if(flag) goto 20
 enddo
enddo

crosslines=.false.

20 continue

deallocate(p1)
deallocate(p2)
deallocate(p3)
deallocate(p4)
deallocate(pc)

end function crosslines


subroutine crosspoint(v1,v2,v3,v4,p,flag)

real*8, intent(in)   :: v1(:),v2(:),v3(:),v4(:)
real*8, intent(out)   :: p(:)
real*8               :: eps,d,t,n1,n2,d1,d2,d3,d4
logical, intent(out)  :: flag

p=0.d0
eps=1.0d-12 
flag=.false.

if(size(v1).gt.2) then
print*,"only 2D up to now"
stop
else
d=scalar((v2-v1)/norm(v2-v1),(v4-v3)/norm(v4-v3)) 
if(d.gt.1.d0-eps.and.d.lt.1.d0+eps) goto 20 
t=-(-v3(1)*v1(2) + v4(1)*v1(2) + v1(1)*v3(2) - v4(1)*v3(2) - &
    v1(1)*v4(2) + v3(1)*v4(2)) / &
   (v3(1)*v1(2) - v4(1)*v1(2) - v3(1)*v2(2) + v4(1)*v2(2) &
    - v1(1)*v3(2) + v2(1)*v3(2) + v1(1)*v4(2) - v2(1)*v4(2))
endif

p=v1+t*(v2-v1)
n1=norm(v2-v1)
n2=norm(v4-v3)
d1=norm(p-v1)
d2=norm(p-v2)
d3=norm(p-v3)
d4=norm(p-v4)
if(d1.le.n1.and.d2.le.n1.and.d3.le.n2.and.d4.le.n2)flag=.true.

20 continue



end subroutine crosspoint

subroutine ruota(vin,p0,fhid,p)  !asse rotazione, punto asse, angolo, punti


integer                 :: npunti,ndim
integer                 :: i,j,k,eps(3,3,3),de(3,3),np
real*8, intent(in)      :: vin(3),p0(3),fhid
real*8, intent(inout)   :: p(:,:)
real*8                  :: sinfi,cosfi,epssum,sumj,a(3,3),fhi,v(3)
real*8, allocatable     :: p1(:,:)
      
  
npunti = size(p(:,1))
ndim   = size(p(1,:))
allocate (p1(npunti,ndim))

            fhi=fhid*(8*datan(1.d0))/360
            v=vin/dsqrt(vin(1)**2+vin(2)**2+vin(3)**2)
     
            eps=0
            eps(1,2,3)=1
            eps(2,3,1)=1
            eps(3,1,2)=1
            eps(3,2,1)=-1
            eps(2,1,3)=-1
            eps(1,3,2)=-1

            de=0
            de(1,1)=1
            de(2,2)=1
            de(3,3)=1

            cosfi=dcos(fhi)
            sinfi=dsin(fhi)


        do i=1,3
         do j=1,3
          epssum=0.
          do k=1,3
           epssum=epssum+eps(i,j,k)*v(k)*sinfi
          enddo
          a(i,j)=de(i,j)*cosfi + v(i)*v(j)*(1-cosfi ) -epssum 
         enddo
        enddo

        do np=1,npunti
          do i=1,3
           sumj=0
            do j=1,3
             sumj=sumj+a(i,j)*(p(np,j)-p0(j)) 
            enddo
           p1(np,i)=sumj + p0(i)
          enddo
        enddo
 p = p1

 deallocate(p1)

 return

end subroutine ruota

real*8 function rmsd(pos1,pos2)

real*8, intent(in)     :: pos1(:,:),pos2(:,:)
integer                :: i

rmsd=0.d0
do i=1,size(pos1(:,1))
 rmsd=rmsd+(pos1(i,1)-pos2(i,1))**2 + (pos1(i,2)-pos2(i,2))**2 &
                  +(pos1(i,3)-pos2(i,3))**2 
enddo
 rmsd=dsqrt(rmsd)

return

end function rmsd

subroutine interpola(p)

real*8, intent(inout)  :: p(:,:,:)
integer                :: n,k
real*8,allocatable     :: vec(:),vec1(:),vec0(:),t(:)
integer                :: na,ndim,i,j
n=size(p(:,1,1))
na=size(p(1,:,1))
ndim=size(p(1,1,:))
!print*,n,na,ndim
allocate(vec(na*ndim))
allocate(vec1(na*ndim))
allocate(vec0(na*ndim))
allocate(t(na*ndim))
do i=1,na
 do j=1,ndim
  vec((i-1)*ndim+j)=p(n,i,j)-p(1,i,j)
  vec0((i-1)*ndim+j)=p(1,i,j)
 enddo
enddo


do i=2,n-1
vec1=vec0
 t=vec*(float(i)-1.d0)/(n-1)
  vec1=vec1+t
 do j=1,na
  do k=1,3
   p(i,j,k)=vec1((j-1)*ndim+k)
  enddo
 enddo
enddo

 return

end subroutine interpola

subroutine trasla(p,t)

real*8, intent(in)     :: t(:)
real*8, intent(inout)  :: p(:,:)

integer                :: i,j


  do i=1,size(p(:,1))
   do j=1,size(t)
     p(i,j)=p(i,j)+t(j)
   enddo
  enddo

 return

end subroutine trasla

subroutine vecprod(v1,v2)

real*8, intent(inout) :: v1(3)
real*8, intent(in)    :: v2(3)
real*8                :: x,y,z

 x = v1(2)*v2(3) - v1(3)*v2(2)
 y = v1(3)*v2(1) - v1(1)*v2(3)
 z = v1(1)*v2(2) - v1(2)*v2(1)

 v1(1) = x
 v1(2) = y
 v1(3) = z

 return

end subroutine vecprod


real*8 function ang(v1,v2)

real*8, intent(in)  :: v1(:),v2(:)
real*8              :: t,cost,de

 pi = datan(1.d0)*4.d0
 de=1.0d-06

       

       cost= scalar(v1,v2)
       cost= cost/(norm(v1)*norm(v2))
       if(cost.gt.1.0d0.and.cost.lt.(1.0d0+de))then
           ang=0.0d0
           goto 33
       endif
       if(cost.lt.-1.d0.and.cost.gt.-1.d0-de)then 
         ang=180.d0
         goto 33
       endif
       t=dacos(cost)
       ang=t*180.0d0/pi
    
 33 continue
 return

end function ang


real*8 function tcos(v1,v2)

real*8, intent(in)  :: v1(:),v2(:)

 pi = datan(1.d0)*4.d0



       tcos= scalar(v1,v2)
       tcos= tcos/(norm(v1)*norm(v2))


 return

end function tcos

subroutine reflex(p,v,p0)

real*8, intent(inout)   :: p(:,:)
real*8, intent(in)      :: v(:),p0(:)
real*8                  :: t(3),fhi,vx(3),vp(3),o(3)
integer                 :: i

 if(v(1) == 0) then
  vx(1) = 1
  vx(2) = 0
  vx(3) = 0
 else
  vx(1) = 0
  vx(2) = 1
  vx(3) = 0
 endif
  o(1) = 0
  o(2) = 0
  o(3) = 0
 vp=v
 t=-p0
 call trasla(p,t)
 fhi = ang(v,vx)
 call vecprod(vp,vx)
 call ruota(vp,o,fhi,p)

 if(v(1) == 0) then
   do i=1,size(p(:,1))
     p(i,2)=-p(i,2)
   enddo 
 else
   do i=1,size(p(:,1))
     p(i,1)=-p(i,1)
   enddo
 endif
 
 fhi = -fhi
 call ruota(vp,o,fhi,p)
 t = -t
 call trasla(p,t)

 return

end subroutine reflex

subroutine cm(g,x)
type(geo), intent(in)       :: g
real*8, intent(out)        :: x(3)
real*8                     :: m,mi
integer                    :: i

x=0.d0
m=0
do i=1,g%n
 mi=ptable(atnum(g%name(i)))%amass * 1822.8884812105385135
 m=m+mi
 x(1)=x(1)+g%p(i,1)*mi
 x(2)=x(2)+g%p(i,2)*mi
 x(3)=x(3)+g%p(i,3)*mi
enddo

x=x/m

end subroutine cm

subroutine inertia(g,v,I)
type(geo), intent(in)       :: g
real*8, intent(in)          :: v(:,:)
real*8, intent(out)         :: I(3,3)
real*8                      :: dumr(3),m,x,y,z
real*8, allocatable         :: r(:,:)
integer                     :: k

I=0

do k=1,g%n
 x=g%p(k,1)
 y=g%p(k,2)
 z=g%p(k,3)
 m=ptable(atnum(g%name(k)))%amass * 1822.8884812105385135
 I(1,1)= I(1,1) + (y*y +z*z)*m
 I(1,2)= I(1,2)  -(x*y)*m
 I(1,3)= I(1,3)  -(x*z)*m
 I(2,2)= I(2,2) + (x*x+z*z)*m
 I(2,3)= I(2,3)  -(y*z)*m
 I(3,3)= I(3,3) + (x*x+y*y)*m
enddo
 I(2,1)=I(1,2)
 I(3,1)=I(1,3)
 I(3,2)=I(2,3)
  

end subroutine inertia

subroutine angular(g,v,I,l,w)

type(geo), intent(in)       :: g
real*8, intent(in)          :: v(:,:),I(3,3)
real*8, intent(out)         :: l(3),w(3)
real*8                      :: xcm(3),dumr(3),x,y,z,II(3,3)
real*8, allocatable         :: r(:,:)
integer                     :: k

l=0
w=0
call cm(g,xcm)

allocate(r(g%n,3))
do k=1,g%n
 r(k,1:3)=g%p(k,1:3)-xcm 
enddo

do k=1,g%n
 dumr=r(k,1:3)
 call vecprod(dumr,v(k,1:3))
 l=l+ptable(atnum(g%name(k)))%amass*1822.8884812105385135*dumr 
enddo

II=I
call inverse(II)
w(1)=II(1,1)*l(1)+II(1,2)*l(2)+II(1,3)*l(3)
w(2)=II(2,1)*l(1)+II(2,2)*l(2)+II(2,3)*l(3)
w(3)=II(3,1)*l(1)+II(3,2)*l(2)+II(3,3)*l(3)

!DSYEV( JOBZ, UPLO, N, A, LDA, W, WORK, LWORK, INFO )

deallocate(r)

end subroutine angular

subroutine inverse(a)
real*8, intent(inout)     :: A(:,:)
integer                   :: N,M,LDA,LWORK,INFO
integer, allocatable      :: IPIV(:)
real*8, allocatable       :: WORK(:)

N=size(A(:,1))
M=N
LDA=N
LWORK=N
allocate(WORK(LWORK))
allocate(IPIV(N))

call DGETRF(M, N, A, LDA, IPIV, INFO)
call DGETRI(N,A,LDA,IPIV,WORK,LWORK,INFO)

deallocate(WORK)
deallocate(IPIV)

end subroutine inverse


subroutine dinverse(a)
double precision, intent(inout)     :: A(:,:)
integer                   :: N,M,LDA,LWORK,INFO
integer, allocatable      :: IPIV(:)
double precision, allocatable       :: WORK(:)

N=size(A(:,1))
M=N
LDA=N
LWORK=N
allocate(WORK(LWORK))
allocate(IPIV(N))

call DGETRF(M, N, A, LDA, IPIV, INFO)
call DGETRI(N,A,LDA,IPIV,WORK,LWORK,INFO)

deallocate(WORK)
deallocate(IPIV)

end subroutine dinverse



real*8 function norm(v)

real*8      :: v(:)
integer     :: i

 norm = 0

 do i=1,size(v)
  norm = norm + v(i)*v(i)
 enddo

 norm =dsqrt(norm)

 return

end function norm

real*8 function scalar(a,b)

 integer                 :: i
 real*8, intent(in)      :: a(:), b(:)

 scalar=0
 do i=1,size(a)
  scalar = scalar + a(i)*b(i)
 enddo
 
 return

end function scalar

subroutine dif(v1,v2,vdif)

real*8, intent(in)     :: v1(:),v2(:)
real*8, intent(out)    :: vdif(:)
integer    :: i

 do i=1,size(v1)
    vdif(i)=v2(i)-v1(i)
 enddo

return

end subroutine dif

subroutine reci(c,cr)
real*8, intent(in)    :: c(:,:)
real*8, intent(out)   :: cr(:,:)
real*8                :: v(3),v1(3),den
pi=4.d0*datan(1.d0)
v=c(2,1:3)
v1=c(3,1:3)
call vecprod(v,v1)
den=scalar(c(1,1:3),v)
cr(1,1:3)=2.d0*pi*v/den
v=c(3,1:3)
v1=c(1,1:3)
call vecprod(v,v1)
cr(2,1:3)=2.d0*pi*v/den
v=c(1,1:3)
v1=c(2,1:3)
call vecprod(v,v1)
cr(3,1:3)=2.d0*pi*v/den
end subroutine reci

subroutine pbc(punti,cella)

real*8,intent(inout)     :: punti(:,:)
real*8,intent(in)        :: cella(3,3)
real*8                   :: s,v(3),p(3)
real*8                   :: det,x,y,z,x1,y1,z1,x2,y2,z2,x3,y3,z3,v1,v2,v3
integer                  :: i,j,ii,case


   x1 = cella(1,1)
   y1 = cella(1,2)
   z1 = cella(1,3)
   x2 = cella(2,1)
   y2 = cella(2,2)
   z2 = cella(2,3)
   x3 = cella(3,1)
   y3 = cella(3,2)
   z3 = cella(3,3)
   det= -x3*y2*z1 + x2*y3*z1 + x3*y1*z2 - x1*y3*z2 - x2*y1*z3 + x1*y2*z3

   do ii=1,size(punti(:,1))
 
   case = 0
   x = punti(ii,1)
   y = punti(ii,2)
   z = punti(ii,3)

   v1= (-x3*y2*z + x2*y3*z + x3*y*z2 - x*y3*z2 - x2*y*z3 + x*y2*z3)/det
   v2= ( x3*y1*z - x1*y3*z - x3*y*z1 + x*y3*z1 + x1*y*z3 - x*y1*z3)/det
   v3= (-x2*y1*z + x1*y2*z + x2*y*z1 - x*y2*z1 - x1*y*z2 + x*y1*z2)/det

   if( v1 >= 1)then
     v1 = v1 - int(v1)
     case=1
   else if(v1<0) then
     v1 = v1 - int(v1) +1
     case=1
   endif

   if( v2 >= 1)then
     v2 = v2 - int(v2)
     case=1
   else if(v2<0) then
     v2 = v2 - int(v2) +1
     case=1
   endif

   if( v3 >= 1)then
     v3 = v3 - int(v3)
     case=1
   else if(v3<0) then
     v3 = v3 - int(v3) +1
     case=1
   endif

   if(case==1)then
     x = v1
     y = v2
     z = v3
     punti(ii,1) = v1*x1 + v2*x2 + v3*x3
     punti(ii,2) = v1*y1 + v2*y2 + v3*y3
     punti(ii,3) = v1*z1 + v2*z2 + v3*z3
   endif

   enddo

 return 

end subroutine pbc

subroutine  vettore(p1,p2,dx,dy,dz,basis)

real*8, intent(in)  :: p1(3),p2(3),basis(3,3)
real*8, intent(out) :: dx,dy,dz
real*8              :: p(2,3),v(2,3)

p(1,1:3)=p1
p(2,1:3)=p2

call cart2aff(p,v,basis) 

if(dabs(v(2,1)-v(1,1)).gt.0.5) then
  if(v(2,1).gt.v(1,1)) then
   v(2,1)=v(2,1)-1.d0
  else
   v(2,1)=1.d0+v(2,1)
  endif
endif

if(dabs(v(2,2)-v(1,2)).gt.0.5) then
  if(v(2,2).gt.v(1,2)) then
   v(2,2)=v(2,2)-1.d0
  else
   v(2,2)=1.d0+v(2,2)
  endif
endif

if(dabs(v(2,3)-v(1,3)).gt.0.5) then
  if(v(2,3).gt.v(1,3)) then
   v(2,3)=v(2,3)-1.d0
  else
   v(2,3)=1.d0+v(2,3)
  endif
endif

call aff2cart(v,p,basis)

dx=p(2,1)-p(1,1)
dy=p(2,2)-p(1,2)
dz=p(2,3)-p(1,3)

return

end subroutine vettore

subroutine neighbors(g,id,nv,v,d)
type(geo), intent(in)   :: g
integer, intent(in)     :: id
integer, intent(inout)  :: v(:)
integer, intent(out)    :: nv
real*8, intent(inout)   :: d(:)
real*8                  :: rcut

integer                 :: i,j,na,nvm
real*8                  :: dt


nv=0
nvm=size(v)
na=g%n
do i=1,id-1
 dt=distanza(g%p(id,1:3),g%p(i,1:3),g%b)
 rcut=(ptable(atnum(g%name(i)))%covalent_radius + ptable(atnum(g%name(id)))%covalent_radius)*1.25d0
 if(dt.le.rcut) then
  nv=nv+1
  if(nv.gt.nvm) then
   print*,"nv greater than nvm"
   stop
  endif
  v(nv)=i
  d(nv)=dt
 endif
enddo
do i=id+1,na
 dt=distanza(g%p(id,1:3),g%p(i,1:3),g%b)
 rcut=(ptable(atnum(g%name(i)))%covalent_radius + ptable(atnum(g%name(id)))%covalent_radius)*1.25d0
 if(dt.le.rcut) then
  nv=nv+1
  if(nv.gt.nvm) then
   print*,"nv gt nvm"
   stop
  endif
  v(nv)=i
  d(nv)=dt
 endif
enddo
return
end subroutine neighbors

subroutine vicini(g,id,nv,v,d,rcut)
type(geo), intent(in)   :: g
integer, intent(in)     :: id
integer, intent(inout)  :: v(:)
integer, intent(out)    :: nv
real*8, intent(in)      :: rcut
real*8, intent(inout)   :: d(:)

integer                 :: i,na,nvm
real*8                  :: dt


nv=0
nvm=size(v)
na=g%n
do i=1,id-1
 dt=distanza(g%p(id,1:3),g%p(i,1:3),g%b)
 if(dt.le.rcut) then
  nv=nv+1
  if(nv.gt.nvm) then
   print*,"nv greater than nvm"
   stop
  endif
  v(nv)=i
  d(nv)=dt
 endif
enddo
do i=id+1,na
 dt=distanza(g%p(id,1:3),g%p(i,1:3),g%b)
 if(dt.le.rcut) then
  nv=nv+1
  if(nv.gt.nvm) then
   print*,"nv gt nvm"
   stop
  endif
  v(nv)=i
  d(nv)=dt
 endif
enddo
return
end subroutine vicini

real*8 function distanza(p1,p2,basis)

real*8, intent(in)   :: p1(3),p2(3),basis(3,3)
real*8               :: dx,dy,dz

call vettore(p1,p2,dx,dy,dz,basis)
distanza=dsqrt(dx*dx +dy*dy +dz*dz)

return

end function distanza

subroutine dx_per(f,nx,ny,nz,dx,dxf)

integer, intent(in) :: nx,ny,nz
real*8, intent(in)  :: f(nx,ny,nz),dx
real*8, intent(out) :: dxf(nx,ny,nz)
integer             :: i,j,k
real*8              :: idx

idx = 1.d0/(2.d0*dx)

do k=1,nz
 do j=1,ny
  do i=2,nx-1
     dxf(i,j,k) = (f(i+1,j,k)-f(i-1,j,k))*idx
  enddo
 enddo
enddo

do k=1,nz
 do j=1,ny
     dxf(1,j,k) = (f(2,j,k)-f(nx,j,k))*idx
 enddo
enddo

do k=1,nz
 do j=1,ny
     dxf(nx,j,k) = (f(1,j,k)-f(nx-1,j,k))*idx
 enddo
enddo

return

end subroutine dx_per

subroutine dy_per(f,nx,ny,nz,dy,dyf)

integer, intent(in) :: nx,ny,nz
real*8, intent(in)  :: f(nx,ny,nz),dy
real*8, intent(out) :: dyf(nx,ny,nz)
integer             :: i,j,k
real*8              :: idy

idy = 1.d0/(2.d0*dy)

do k=1,nz
 do i=1,nx
  do j=2,ny-1
     dyf(i,j,k) = (f(i,j+1,k)-f(i,j-1,k))*idy
  enddo
 enddo
enddo

do k=1,nz
 do i=1,nx
     dyf(i,1,k) = (f(i,2,k)-f(i,ny,k))*idy
 enddo
enddo

do k=1,nz
 do i=1,nx
     dyf(i,ny,k) = (f(i,1,k)-f(i,ny-1,k))*idy
 enddo
enddo

return

end subroutine dy_per

subroutine dz_per(f,nx,ny,nz,dz,dzf)

integer, intent(in) :: nx,ny,nz
real*8, intent(in)  :: f(nx,ny,nz),dz
real*8, intent(out) :: dzf(nx,ny,nz)
integer             :: i,j,k
real*8              :: idz

idz = 1.d0/(2.d0*dz)

do j=1,ny
 do i=1,nx
  do k=2,nz-1
     dzf(i,j,k) = (f(i,j,k+1)-f(i,j,k-1))*idz
  enddo
 enddo
enddo

do j=1,ny
 do i=1,nx
     dzf(i,j,1) = (f(i,j,2)-f(i,j,nz))*idz
 enddo
enddo

do j=1,ny
 do i=1,nx
     dzf(i,j,nz) = (f(i,j,1)-f(i,j,nz-1))*idz
 enddo
enddo

return

end subroutine dz_per

!subroutine lap_per(f,nx,ny,nz,d2f)
!
!real*8, intent(in)    :: f(0:nx-1,0:ny-1,0:nz-1)
!real*8, intent(out)   :: d2f(0:nx-1,0:ny-1,0:nz-1)
!real*8                :: ft(0:nx-1,0:ny-1,0:nz-1),scale
!integer               :: isign,nx,ny,nz,aux(62000),naux

!naux=62000
!if(nx.ge.252.or.ny.ge.252.or.nz.ge.252)then
!print*,"cambiare naux"
!stop
!endif
!
!isign=1
!call drcft3(f,nx,nx*ny,ft,nx,nx*ny,nx,ny,nz,isign,scale,aux,naux)



!return

!end subroutine lap_per

subroutine ijk(i,xx,dx)

integer, intent(out)  :: i(3)
real*8, intent(in)    :: dx(3,3)
real*8, intent(in)    :: xx(3)
real*8                ::a1,a2,a3,b1,b2,b3,c1,c2,c3,x,y,z,norm1,norm2,norm3
real*8                :: v1(3),v2(3),v3(3)

x=xx(1)
y=xx(2)
z=xx(3)

a1=dx(1,1)
a2=dx(1,2)
a3=dx(1,3)
!
b1=dx(2,1)
b2=dx(2,2)
b3=dx(2,3)
!
c1=dx(3,1)
c2=dx(3,2)
c3=dx(3,3)
!norm1=norm(dx(1,1:3))
!norm2=norm(dx(2,1:3))
!norm3=norm(dx(3,1:3))
!v1=dx(1,1:3)/norm1
!v2=dx(2,1:3)/norm2
!v3=dx(3,1:3)/norm3

!i(1)=nint(scalar(v1,xx)/norm1)+1
!i(2)=nint(scalar(v2,xx)/norm2)+1
!i(3)=nint(scalar(v3,xx)/norm3)+1


    i(1) = 1+nint( + &
         (-(b3*c2*x) + b2*c3*x + b3*c1*y - b1*c3*y - b2*c1*z + &
            b1*c2*z)/ &
          (-(a3*b2*c1) + a2*b3*c1 + a3*b1*c2 - a1*b3*c2 - a2*b1*c3 + &
            a1*b2*c3)) 
    i(2)=   1+nint( + &
         (-(a3*c2*x) + a2*c3*x + a3*c1*y - a1*c3*y - a2*c1*z + &
            a1*c2*z)/ &
          (a3*b2*c1 - a2*b3*c1 - a3*b1*c2 + a1*b3*c2 + a2*b1*c3 -  &
            a1*b2*c3)) 
    i(3)=   1+nint( + &
         (-(a3*b2*x) + a2*b3*x + a3*b1*y - a1*b3*y - a2*b1*z + &
            a1*b2*z)/ &
          (-(a3*b2*c1) + a2*b3*c1 + a3*b1*c2 - a1*b3*c2 - a2*b1*c3 + &
            a1*b2*c3)) 
!    if(x.lt.-0.0)

return

end subroutine ijk


subroutine xyz(ii,x,dx)

integer, intent(in)  :: ii(3)
real*8, intent(in)   :: dx(3,3)
real*8, intent(out)  :: x(3)
real*8               ::a1,a2,a3,b1,b2,b3,c1,c2,c3
integer              :: i,j,k

i=ii(1)
j=ii(2)
k=ii(3)

a1=dx(1,1)
a2=dx(1,2)
a3=dx(1,3)

b1=dx(2,1)
b2=dx(2,2)
b3=dx(2,3)

c1=dx(3,1)
c2=dx(3,2)
c3=dx(3,3)

x(1)=  a1*(-1 + i) + b1*(-1 + j) + c1*(-1 + k)
x(2)=  a2*(-1 + i) + b2*(-1 + j) + c2*(-1 + k)
x(3)=  a3*(-1 + i) + b3*(-1 + j) + c3*(-1 + k)

return

end subroutine xyz


subroutine grad_point(f,v2,dxx,gf)

real*8, intent(in)  :: dxx(3,3),f(-2:2,-2:2,-2:2)
real*8, intent(out) :: gf(3)
integer, intent(in) :: v2(3)
real*8              :: idx,idy,idz,da,db,dc,n1,n2,n3,id1,id2,id3
real*8              :: a1,a2,a3,b1,b2,b3,c1,c2,c3
integer             :: i,j,k,il,ir,jl,jr,kl,kr


n1=norm(dxx(1,1:3))
n2=norm(dxx(2,1:3))
n3=norm(dxx(3,1:3))
a1=dxx(1,1)
a2=dxx(1,2)
a3=dxx(1,3)
b1=dxx(2,1)
b2=dxx(2,2)
b3=dxx(2,3)
c1=dxx(3,1)
c3=dxx(3,2)
c3=dxx(3,3)
id1=1.d0/(2.d0*n1)
id2=1.d0/(2.d0*n2)
id3=1.d0/(2.d0*n3)

i=v2(1)
j=v2(2)
k=v2(3)
    il=i-1
    ir=i+1
    jl=j-1
    jr=j+1
    kl=k-1
    kr=k+1

    da=(f(ir,j,k)-f(il,j,k))*id1
    db=(f(i,jr,k)-f(i,jl,k))*id2
    dc=(f(i,j,kr)-f(i,j,kl))*id3

gf(1)=       (-(b3*c2*da*N1) + b2*c3*da*N1 + b3*c1*db*N1 - &
                  b1*c3*db*N1 - b2*c1*dc*N1 + b1*c2*dc*N1)/ &
                  (-(a3*b2*c1) + a2*b3*c1 + a3*b1*c2 - a1*b3*c2 - a2*b1*c3 +  &
                  a1*b2*c3)
gf(2)=(-(a3*c2*da*N2) + a2*c3*da*N2 + a3*c1*db*N2 - &
           a1*c3*db*N2 - a2*c1*dc*N2 + a1*c2*dc*N2)/ &
         (a3*b2*c1 - a2*b3*c1 - a3*b1*c2 + a1*b3*c2 + a2*b1*c3 - &
           a1*b2*c3)
gf(3)=-(((a3*b2*da - a2*b3*da - a3*b1*db + a1*b3*db + &
               a2*b1*dc - a1*b2*dc)*N3)/ &
           (-(a3*b2*c1) + a2*b3*c1 + a3*b1*c2 - a1*b3*c2 - a2*b1*c3 + &
             a1*b2*c3))



return
end subroutine grad_point

subroutine grad(f,dxx,gf)

real*8, intent(in)  :: f(:,:,:)
real*8, intent(in)  :: dxx(3,3)
real*8, intent(out) :: gf(:,:,:,:)
real*8              :: idx,idy,idz,da,db,dc,n1,n2,n3,id1,id2,id3
real*8              :: a1,a2,a3,b1,b2,b3,c1,c2,c3
integer             :: nx,ny,nz,i,j,k,il,ir,jl,jr,kl,kr

nx=size(f(:,1,1))
ny=size(f(1,:,1))
nz=size(f(1,1,:))
!idx=1.d0/(2.d0*dxx(1,1))
!idy=1.d0/(2.d0*dxx(2,2))
!idz=1.d0/(2.d0*dxx(3,3))
n1=norm(dxx(1,1:3))
n2=norm(dxx(2,1:3))
n3=norm(dxx(3,1:3))
a1=dxx(1,1)
a2=dxx(1,2)
a3=dxx(1,3)
b1=dxx(2,1)
b2=dxx(2,2)
b3=dxx(2,3)
c1=dxx(3,1)
c3=dxx(3,2)
c3=dxx(3,3)
id1=1.d0/(2.d0*n1)
id2=1.d0/(2.d0*n2)
id3=1.d0/(2.d0*n3)

do i=1,nx
    il=i-1
    ir=i+1
    if(il.eq.0)il=nx
    if(ir.eq.nx+1)ir=1
 do j=1,ny
    jl=j-1
    jr=j+1
    if(jl.eq.0)jl=ny
    if(jr.eq.ny+1)jr=1
  do k=1,nz
    kl=k-1
    kr=k+1
    if(kl.eq.0)kl=nz
    if(kr.eq.nz+1)kr=1

    da=(f(ir,j,k)-f(il,j,k))*id1
    db=(f(i,jr,k)-f(i,jl,k))*id2
    dc=(f(i,j,kr)-f(i,j,kl))*id3

gf(i,j,k,1)=       (-(b3*c2*da*N1) + b2*c3*da*N1 + b3*c1*db*N1 - &
                  b1*c3*db*N1 - b2*c1*dc*N1 + b1*c2*dc*N1)/ &
                  (-(a3*b2*c1) + a2*b3*c1 + a3*b1*c2 - a1*b3*c2 - a2*b1*c3 +  &
                  a1*b2*c3)
gf(i,j,k,2)=(-(a3*c2*da*N2) + a2*c3*da*N2 + a3*c1*db*N2 - &
           a1*c3*db*N2 - a2*c1*dc*N2 + a1*c2*dc*N2)/ &
         (a3*b2*c1 - a2*b3*c1 - a3*b1*c2 + a1*b3*c2 + a2*b1*c3 - &
           a1*b2*c3)
gf(i,j,k,3)=-(((a3*b2*da - a2*b3*da - a3*b1*db + a1*b3*db + &
               a2*b1*dc - a1*b2*dc)*N3)/ &
           (-(a3*b2*c1) + a2*b3*c1 + a3*b1*c2 - a1*b3*c2 - a2*b1*c3 + &
             a1*b2*c3))

!   gf(i,j,k,1)=der1*dxx(1,1)/norm1 + der2*dxx(2,1)/norm2 + der3*dxx(3,1)/norm3
!   gf(i,j,k,2)=der1*dxx(1,2)/norm1 + der2*dxx(2,2)/norm2 + der3*dxx(3,2)/norm3
!   gf(i,j,k,3)=der1*dxx(1,3)/norm1 + der2*dxx(2,3)/norm2 + der3*dxx(3,3)/norm3

!   gf(i,j,k,1)=(f(ir,j,k)-f(il,j,k))*idx
!   gf(i,j,k,2)=(f(i,jr,k)-f(i,jl,k))*idy
!   gf(i,j,k,3)=(f(i,j,kr)-f(i,j,kl))*idz

  enddo
 enddo
enddo


return

end subroutine grad

logical function zero1d(f,n)

integer, intent(in) :: n
real*8,  intent(in) :: f(n-1:n+1)
real*8              :: m
integer             :: n0
logical             :: cs

zero1d=.false.
if(f(n).eq.0.d0) then
  zero1d=.true.
else
  cs=(f(n-1).lt.0.d0.and.f(n+1).gt.0.d0).or. &
     (f(n+1).lt.0.d0.and.f(n-1).gt.0.d0)
  if(cs) then
    m=(f(n+1)-f(n-1))/2.d0
    n0=nint(float(n)-f(n)/m)
    if(n0.eq.n)zero1d=.true.
  endif
endif

return

end function zero1d  

!logical function zero3d(f,n1,n2,n3)

!real*8,  intent(in)  ::f(n1-1:n1+1,n2-1:n2+1,n3-1:n3+1)
!integer, intent(in)  :: n1,n2,n3

!zero3d=zero1d(f(n1-1:n1+1,n2,n3).or.zero1d(f(n1,n2-1:n2+1,n3),n2) &
!       .or. zero1d(f(n1,n2,n3-1:n3+1),n3)

!return

!end function zero3d

subroutine lap(f,dx,dy,dz,lapf)

real*8, intent(in)  :: f(:,:,:)
real*8, intent(in)  :: dx,dy,dz
real*8, intent(out) :: lapf(:,:,:)
real*8              :: idx2,idy2,idz2,lapx,lapy,lapz
integer             :: nx,ny,nz,i,j,k,il,ir,jl,jr,kl,kr

nx=size(f(:,1,1))
ny=size(f(1,:,1))
nz=size(f(1,1,:))
idx2=1.d0/(dx*dx)
idy2=1.d0/(dy*dy)
idz2=1.d0/(dz*dz)

do i=1,nx
 il=i-1
 ir=i+1
 if(i.eq.0) il=nx  
 if(i.eq.nx)ir=1  
 do j=1,ny
  jl=j-1
  jr=j+1
  if(j.eq.0) jl=ny  
  if(j.eq.ny)jr=1  
  do k=1,nz
   kl=k-1
   kr=k+1
   if(k.eq.0) kl=nz  
   if(k.eq.nz)kr=1  
     lapx=f(ir,j,k)+f(il,j,k)-2.d0*f(i,j,k)
     lapy=f(i,jr,k)+f(i,jl,k)-2.d0*f(i,j,k)
     lapz=f(i,j,kr)+f(i,j,kl)-2.d0*f(i,j,k)
     lapf(i,j,k)=lapx*idx2+lapy*idy2+lapz*idz2
  enddo
 enddo
enddo



end subroutine lap

subroutine cart2aff(posc,posa,basis)   
real*8, intent(in)   :: basis(3,3),posc(:,:)
real*8, intent(out)  :: posa(:,:)
real*8               :: x,y,z,x1,y1,z1,x2,y2,z2,x3,y3,z3,det
integer              :: i

   x1 = basis(1,1)
   y1 = basis(1,2)
   z1 = basis(1,3)
   x2 = basis(2,1)
   y2 = basis(2,2)
   z2 = basis(2,3)
   x3 = basis(3,1)
   y3 = basis(3,2)
   z3 = basis(3,3)
   det= -x3*y2*z1 + x2*y3*z1 + x3*y1*z2 - x1*y3*z2 - x2*y1*z3 + x1*y2*z3
   det=1.d0/det

   do i=1,size(posc(:,1))

   x=posc(i,1)
   y=posc(i,2)
   z=posc(i,3)

   posa(i,1)= (-x3*y2*z + x2*y3*z + x3*y*z2 - x*y3*z2 - x2*y*z3 + x*y2*z3)*det
   posa(i,2)= ( x3*y1*z - x1*y3*z - x3*y*z1 + x*y3*z1 + x1*y*z3 - x*y1*z3)*det
   posa(i,3)= (-x2*y1*z + x1*y2*z + x2*y*z1 - x*y2*z1 - x1*y*z2 + x*y1*z2)*det

   enddo
   
   return

end subroutine cart2aff

subroutine aff2cart(posa,posc,basis)
real*8, intent(in)   :: basis(3,3),posa(:,:)
real*8, intent(out)  :: posc(:,:)
real*8               :: x,y,z,x1,y1,z1,x2,y2,z2,x3,y3,z3,v1,v2,v3
integer              :: i

   do i=1,size(posa(:,1))

   v1=posa(i,1)
   v2=posa(i,2)
   v3=posa(i,3)

   posc(i,1)=v1*basis(1,1)+v2*basis(2,1)+v3*basis(3,1)
   posc(i,2)=v1*basis(1,2)+v2*basis(2,2)+v3*basis(3,2)
   posc(i,3)=v1*basis(1,3)+v2*basis(2,3)+v3*basis(3,3)

   enddo
   
   return

end subroutine aff2cart

subroutine basis2celldm(basis,a,b,c,alpha,beta,gamma)

real*8, intent(in)  :: basis(3,3)
real*8, intent(out) :: a,b,c,alpha,beta,gamma

a=norm(basis(1,1:3))
b=norm(basis(2,1:3))/a
c=norm(basis(3,1:3))/a

alpha=tcos(basis(2,1:3),basis(3,1:3))
beta=tcos(basis(1,1:3),basis(3,1:3))
gamma=tcos(basis(1,1:3),basis(2,1:3))

return

end subroutine basis2celldm

subroutine celldm2basis(celldm,basis)

real*8, intent(in)      :: celldm(6)
real*8, intent(out)      :: basis(3,3)
real*8                  :: a,b,c,cosxy,cosxz,cosyz

pi=datan(1.d0)*4.d0
basis=0
a=celldm(1)
b=celldm(2)
c=celldm(3)
cosyz=dcos(celldm(4)*pi/180.d0)
cosxz=dcos(celldm(5)*pi/180.d0)
cosxy=dcos(celldm(6)*pi/180.d0)
basis(1,1)=a
basis(2,1)=b*cosxy
basis(2,3)=b*cosyz
basis(2,2)=dsqrt(b*b-basis(2,1)**2-basis(2,3)**2)
basis(3,1)=c*cosxz
basis(3,2)=c*cosyz
basis(3,3)=dsqrt(c*c-basis(3,1)**2-basis(3,2)**2)


return

end subroutine celldm2basis

logical recursive function connected(g,followed,id1,id2) result(conne)

type(geo), intent(in)       :: g
integer, intent(in)         :: id1,id2
logical, intent(inout)      :: followed(:)
real*8                      :: bd,dd,d(12)
integer                     :: i,nv,v(12)

conne=.false.
followed(id1)=.true.

call neighbors(g,id1,nv,v,d)

do i=1,nv
 if(v(i).eq.id2) then
  conne=.true.
  goto 30
 else
  if( .not.followed(v(i)) ) then
   followed(v(i))=.true.
   if(connected(g,followed,v(i),id2)) then
    conne=.true.
    goto 30
   endif
  endif
 endif
enddo

30 continue

return

end function connected
 
integer recursive function co(ng,n,v,vold,vtot,total) result(dum)


!ng = number of elements
!n  = how many elements per time
!vold work vector dimension to ?0
!vtot contains the combination groups TO BE DEALLOCATED
!total= number of combinations


!saple programr:
!program p
!
!use operations
!
!implicit none
!
!integer              :: i,vold(0),ng,n,total,dum
!integer, allocatable :: v(:),vtot(:,:)
!
!allocate(v(4))
!
!v(1)=1
!v(2)=2
!v(3)=3
!v(4)=4
!n=3
!ng=4
!
!dum=co(ng,n,v,vold,vtot,total)
!print*,total
!
!do i=1,total
!   print*,vtot(i,1:n)
!enddo
!
!end

integer, intent(in)  :: ng,n
integer, intent(in)  :: v(:),vold(:)
integer,allocatable, intent(inout) :: vtot(:,:)
integer, allocatable :: v2(:),vnew(:)
!integer, save             :: called
integer, intent(inout)    :: total
integer  :: num,nres,i
integer  :: tot

!print*,called

if(.not.allocated(vtot))then
total=0
!if(called.eq.0)then
tot=ng-n+1
do i=ng-n+2,ng
tot=tot*i
enddo
do i=2,n
tot=tot/i
enddo
!do i=2,ng-n
!tot=tot/i
!enddo
allocate(vtot(tot,n))
total=0
endif
!called=called+1


if (n==1) then
do i=1,ng
total=total+1
!write(6,*)vold,v(i)
vtot(total,1:size(vold))=vold
vtot(total,size(vold)+1)=v(i)
enddo
return

else
num=0
nres=ng-1
do while(nres.ge.n-1)

num=num+1
!print*,"num",num,"size",size(vold)
allocate(vnew(size(vold)+1))
if(size(vold).gt.0)vnew(1:size(vold))=vold
vnew(size(vold)+1)=v(num)
!print*,"vnew",vnew
allocate(v2(ng-num))
v2(1:ng-num)=v(num+1:ng)
!print*,"v2",v2
dum=co(ng-num,n-1,v2,vnew,vtot,total)
nres=nres-1
!print*,"nres",nres
deallocate(vnew)
deallocate(v2)
enddo

endif

!eallocate(vtot)

end function co 

subroutine hpsort(n, ra, ind)         !!da quantum espresso

  !---------------------------------------------------------------------
  ! sort an array ra(1:n) into ascending order using heapsort algorithm.
  ! n is input, ra is replaced on output by its sorted rearrangement.
  ! create an index table (ind) by making an exchange in the index array
  ! whenever an exchange is made on the sorted data array (ra).
  ! in case of equal values in the data array (ra) the values in the
  ! index array (ind) are used to order the entries.
  ! if on input ind(1)  = 0 then indices are initialized in the routine,
  ! if on input ind(1) != 0 then indices are assumed to have been
  !                initialized before entering the routine and these
  !                indices are carried around during the sorting process
  !
  ! no work space needed !
  ! free us from machine-dependent sorting-routines !
  !
  ! adapted from Numerical Recipes pg. 329 (new edition)
  !
!!use kinds
  implicit none
  !-input/output variables
  integer :: n
  integer :: ind (n)
  real*8   :: ra (n)
  !-local variables
  integer :: i, ir, j, l, iind
  real*8   :: rra
  ! initialize index array
  if (ind (1) .eq.0) then
     do i = 1, n
        ind (i) = i
     enddo
  endif
  ! nothing to order
  if (n.lt.2) return
  ! initialize indices for hiring and retirement-promotion phase
  l = n / 2 + 1
  ir = n
10 continue
  ! still in hiring phase
  if (l.gt.1) then
     l = l - 1
     rra = ra (l)
     iind = ind (l)
     ! in retirement-promotion phase.
  else
     ! clear a space at the end of the array
     rra = ra (ir)
     !
     iind = ind (ir)
     ! retire the top of the heap into it
     ra (ir) = ra (1)
     !
     ind (ir) = ind (1)
     ! decrease the size of the corporation
     ir = ir - 1
     ! done with the last promotion
     if (ir.eq.1) then
        ! the least competent worker at all !
        ra (1) = rra
        !
        ind (1) = iind
        return
     endif
  endif
  ! wheter in hiring or promotion phase, we
  i = l
  ! set up to place rra in its proper level
  j = l + l
  !
  do while (j.le.ir)
     if (j.lt.ir) then
        ! compare to better underling
        if (ra (j) .lt.ra (j + 1) ) then
           j = j + 1
        elseif (ra (j) .eq.ra (j + 1) ) then
           if (ind (j) .lt.ind (j + 1) ) j = j + 1
        endif
     endif
     ! demote rra
     if (rra.lt.ra (j) ) then
        ra (i) = ra (j)
        ind (i) = ind (j)
        i = j
        j = j + j
     elseif (rra.eq.ra (j) ) then
        ! demote rra
        if (iind.lt.ind (j) ) then
           ra (i) = ra (j)
           ind (i) = ind (j)
           i = j
           j = j + j
        else
           ! set j to terminate do-while loop
           j = ir + 1
        endif
        ! this is the right place for rra
     else
        ! set j to terminate do-while loop
        j = ir + 1
     endif
  enddo
  ra (i) = rra
  ind (i) = iind
  goto 10
  !

end subroutine hpsort

subroutine sortindex(a,b)

real*8, intent(in)       :: b(:)
integer, intent(inout)   :: a(:)
real*8,allocatable       :: c(:)         
integer                  :: i,in(1)


allocate(c(size(a)))
c=b
do i=1,size(a)
 in=minloc(c)
 a(i)=in(1)
 c(a(i))=100000000000000000.d0
enddo

deallocate(c)

end subroutine sortindex

integer function sign(i)

integer, intent(in)   ::i

if(i.gt.0)then
  sign=1
else if(i.eq.0)then
  sign=0
else
 sign=-1
endif

return

end function sign

real*8 function rsign(i)

real*8, intent(in)   ::i

if(i.gt.0)then
  rsign=1
else if(i.eq.0)then
  rsign=0
else
 rsign=-1
endif

return

end function rsign

subroutine pbci(p,nx,ny,nz)

integer, intent(inout)    :: p(3)
integer, intent(in)       :: nx,ny,nz

if(p(1).gt.nx)p(1)=p(1)-nx
if(p(1).lt.1) p(1)=p(1)+nx
if(p(2).gt.ny)p(2)=p(2)-ny
if(p(2).lt.1) p(2)=p(2)+ny
if(p(3).gt.nz)p(3)=p(3)-nz
if(p(3).lt.1) p(3)=p(3)+nz

return
end subroutine pbci

subroutine vettorei(v1,v2,i1,i2,i3,nx,ny,nz)
integer, intent(in)       :: v1(3),v2(3),nx,ny,nz
integer, intent(out)      :: i1,i2,i3

i1=v2(1)-v1(1)
i2=v2(2)-v1(2)
i3=v2(3)-v1(3)

if(i1.gt.nx/2)i1=i1-nx
if(i1.lt.-nx/2)i1=i1+nx
if(i2.gt.ny/2)i2=i2-ny
if(i2.lt.-ny/2)i2=i2+ny
if(i3.gt.nz/2)i3=i3-nz
if(i3.lt.-nz/2)i3=i3+nz

return
end subroutine vettorei

real*8 function nlm(l,m)

integer, intent(in)   :: l,m
integer               :: k,i,j
real*8                :: a,pi

pi=4.d0*datan(1.d0)
i=l  ! for (int i=0; i<=l; i++)
j=m  !	for (int j=0; j<=i; j++)
		
		a = 1.d0
	        do k=i-j+1,i+j !	for (k=i-j+1; k<=i+j; k++)
                 a=a*k 	       !	A *= (double)k;
                enddo
		nlm = dsqrt((2.0*i+1.0)/4.0/pi/A)


return
end function nlm

real*8 function plm(l,m,x)

integer, intent(in)   :: l,m
integer               :: i,ll
real*8, intent(in)    :: x
real*8                :: pmm,somx2,pmmp1,fact,pll

		if(dabs(x).gt.1)  then     !   if (fabs(x[k])>1.0)
		plm=0.d0   !	legPolValues[(l*(l+1)/2+m)*numberOfDifferentXs+k] = 0.0;
		else

				pmm=1.0
				if (m.gt.0)  then

					somx2=dsqrt(1-x*x)!	somx2=sqrt((1.0-x[k])*(1.0+x[k]));
						fact=1.0
						do i=1,m !for (i=1; i<=m; i++)
						
						pmm=-pmm*fact*somx2 !	pmm*=-fact*somx2;
								fact=fact+2.d0
						enddo
				endif
				if (l.eq.m) then
						plm=pmm ! legPolValues[(l*(l+1)/2+m)*numberOfDifferentXs+k] = pmm;
				else
				
						pmmp1=x*(2*m+1)*pmm
						if (l==(m+1)) then
						plm=pmmp1 !		legPolValues[(l*(l+1)/2+m)*numberOfDifferentXs+k] = pmmp1;
						else
						
						do ll=m+2,l !	for(ll=m+2; ll<=l; ll++)
								
										pll=(x*(2*ll-1)*pmmp1-(ll+m-1)*pmm)/(ll-m)
										pmm=pmmp1
										pmmp1=pll
					        enddo			
								plm = pll
						endif
				endif
		endif

return
end function plm

complex*16  function ylm(l,m,t,f)

real*8, intent(in)   :: t,f
integer, intent(in)   :: l,m
if(m.ge.0) then
 ylm=nlm(l,m)*plm(l,m,cos(t))*(cos(m*f)+cmplx(0.d0,1.d0)*sin(m*f))
else
 ylm=expo(-m)*nlm(l,-m)*plm(l,-m,cos(t))*(cos(-m*f)+cmplx(0.d0,-1.d0)*sin(-m*f))
endif

end function ylm

real*8 function rylm(l,m,t,f)

real*8, intent(in)   :: t,f
integer, intent(in)   :: l,m

if(m.gt.0)then
 rylm=dsqrt(2.d0)*nlm(l,m)*plm(l,m,cos(t))*cos(m*f)
 else if(m.eq.0) then
 rylm=real(ylm(l,m,t,f))
 else
 rylm=dsqrt(2.d0)*nlm(l,-m)*plm(l,-m,cos(t))*sin(m*f)
endif

end function rylm


integer function expo(m)

integer, intent(in)   :: m
expo=-1
if(mod(m,2).eq.0)expo=1
return
end function expo


real*8 function rc(x,f)

real*8, intent(in)         :: x,f(:,:)
real*8                     ::d1,d2,der
logical                    :: found
integer                    :: i,j,k,s,position,divide,counter
integer                    :: lower,upper

s=size(f(:,1))

found = .false.
counter=0

if(x.le.f(1,1)) then
 rc=f(1,2)
 found=.true.
else if(x.ge.f(s,1)) then
 rc=f(s,2)
 found=.true.
else

 divide=2
 position=s/divide
 lower=1
 upper=s
 do while(.not.found)

  counter=counter+1
  if(counter.gt.s/2)then 
    stop
  endif
  if(x.eq.f(position,1))then
   rc=f(position,2)
   found=.true.
!  write(6,33)"x eq p   ",position,x,f(position,1)

  else  if(x.gt.f(position+1,1))then
   lower=position
   position=position+(upper-position)/2
!  write(6,33)"x gt p+1 ",position,x,f(position,1)
 
  else if(x.lt.f(position-1,1))then
   upper=position
   position=position-(position-lower)/2
!  write(6,33)"x lt p-1 ",position,x,f(position,1)
 
  else  if(x.gt.f(position,1)) then
   der = (f(position+1,2)-f(position,2))/(f(position+1,1)-f(position,1))
   rc  = f(position,2)+der*(x-f(position,1))
   found=.true.
!  write(6,33),"x gt p   ",position,x,f(position,1)
  else if (x.lt.f(position,1)) then
   der = (f(position,2)-f(position-1,2))/(f(position,1)-f(position-1,1))
   rc  = f(position-1,2)+der*(x-f(position-1,1))
   found=.true.
!  print*,"x lt p   ",position,x,f(position,1)
  endif

 enddo

33 format(a9,i5,2f10.4)

endif


end function rc


real*8 function oldrc(x,f)

real*8, intent(in)         :: x,f(:,:)
real*8                     ::d1,d2,der
integer :: i,j,k,s

s=size(f(:,1))
if(x.lt.f(1,1)) then
 oldrc=f(1,2)
 goto 20
endif
if(x.gt.f(s,1)) then
 oldrc=f(s,2)
 goto 20
endif

do i=1,s-1
 if(x.eq.f(i,1))then
  oldrc=f(i,2)
  goto 20
 endif
 if(x.eq.f(i+1,1))then
  oldrc=f(i+1,2)
  goto 20
 endif
 if(x.gt.f(i,1).and.x.lt.f(i+1,1)) then
  der = (f(i+1,2)-f(i,2))/(f(i+1,1)-f(i,1))
  oldrc  = f(i,2)+der*(x-f(i,1))
  goto 20
 endif

enddo


20 continue
return
end function oldrc



real*8 function drc(x,f)

real*8, intent(in)         :: x,f(:,:)
real*8                     ::d1,d2,der
integer :: i,j,k,s

s=size(f(:,1))
if(x.le.f(1,1)) then
 drc=f(2,1)-f(1,1)
 goto 20
endif
if(x.gt.f(s,1)) then
 drc=f(s,1)-f(s-1,1)
 goto 20
endif

do i=2,s-2
 if(x.eq.f(i,1))then
  drc=(f(i+1,1)-f(i-1,1))/2.d0
  goto 20
 endif
 if(x.eq.f(i+1,1))then
  drc=(f(i+2,1)-f(i,1))/2.d0
  goto 20
 endif
 if(x.gt.f(i,1).and.x.lt.f(i+1,1)) then
  drc  = f(i+1,1)-f(i,1)
  goto 20
 endif

enddo


20 continue
return
end function drc






real*8 function gau_nd(x,x0,s)

real*8, intent(in)   :: x(:),x0(:),s(:)
real*8               :: pi
integer              :: i
pi=4.d0*datan(1.d0)

gau_nd=1.d0

do i=1,size(x)
gau_nd=gau_nd*gau(x(i),x0(i),s(i))
enddo

return

end function gau_nd

real*8 function gau(x,x0,s)

real*8, intent(in)   :: x,x0,s
real*8               :: pi
pi=4.d0*datan(1.d0)

gau=1.d0/s/dsqrt(2.d0*pi)*dexp(-(x-x0)*(x-x0)/2.d0/s/s)

return
end function gau

real*8 function lore(x,x0,s)
real*8, intent(in)   :: x,x0,s
real*8               :: pi
pi=4.d0*datan(1.d0)

lore=1.d0/pi * 0.5d0*s/( (x-x0)**2 +0.025d0*s*s )

return
end function lore

real*8 function lennard(a,s,r)

real*8, intent(in)  :: a,s,r

lennard=a*( (s/r)**12 -2.d0*(s/r)**6)

return

end function lennard

subroutine par3p(p,coe)

real*8, intent(in)    :: p(3,2)
real*8, intent(out)   :: coe(3)
real*8                :: y1,x1,y2,x2,y3,x3


y1=p(1,2)
x1=p(1,1)
y2=p(2,2)
x2=p(2,1)
y3=p(3,2)
x3=p(3,1)

coe(1)= -((-(x2*y1) + x3*y1 + x1*y2 - x3*y2 -  x1*y3 + x2*y3)/   &
            ((x2 - x3)*  (x1**2 - x1*x2 - x1*x3 + x2*x3)))
coe(2)=  -((x2**2*y1 - x3**2*y1 - x1**2*y2 + &
              x3**2*y2 + x1**2*y3 - x2**2*y3)/ &
            ((x1 - x2)*(x1 - x3)*(x2 - x3)))
coe(3)=  -((-(x2**2*x3*y1) + x2*x3**2*y1 +  &
              x1**2*x3*y2 - x1*x3**2*y2 -  &
              x1**2*x2*y3 + x1*x2**2*y3)/ &
            ((x2 - x3)* &
              (x1**2 - x1*x2 - x1*x3 + x2*x3)))

return

end subroutine par3p

real*8 function xparab(y,x0,coeff)

real*8, intent(in)    :: coeff(3),y,x0
real*8                :: x1,x2
if(abs(coeff(1)).gt.0.d0)then

x1= (-coeff(2) - dsqrt( coeff(2)*coeff(2)-4.d0*coeff(1)*(coeff(3)-y) ) )/(2.d0*coeff(1))
x2= (-coeff(2) + dsqrt( coeff(2)*coeff(2)-4.d0*coeff(1)*(coeff(3)-y) ) )/(2.d0*coeff(1))

xparab=x1
if(dabs(x2-x0).lt.dabs(x1-x0))xparab=x2
else
xparab=(y-coeff(3))/coeff(2)
endif

end function xparab

subroutine nearest_atom(x,pos,basis,theatom,d)
real*8, intent(in)     :: pos(:,:),x(3),basis(3,3)
real*8, intent(out)    :: d(:)
integer, intent(out)   :: theatom(:)
real*8                 :: dnow
integer                :: i,j,n,m

n=size(pos(:,1))
m=size(theatom)
d=5000

do i=1,n
 dnow=distanza(x,pos(i,1:3),basis)
 if(dnow.lt.d(1))then
  do j=m,2,-1
   d(j)=d(j-1) 
   theatom(j)=theatom(j-1)
  enddo
  d(1)=dnow 
  theatom(1)=i
 endif
enddo

return

end subroutine nearest_atom

logical function pari(m)
integer, intent(in)     :: m
pari=.false.
if((m/2)*2.eq.m) pari=.true.
return
end function pari

subroutine cart2spher(rcc,rs)
real*8, intent(in)   :: rcc(3)
real*8, intent(out)   :: rs(3)
real*8                :: eps

pi=datan(1.d0)*4.d0
eps=1.d-15

rs=0.d0
rs(1)=norm(rcc)
if(rs(1).gt.eps) then    !else origin

 rs(2)=dacos(rcc(3)/rs(1))  ! theta

 if(rcc(1).gt.eps) then
  rs(3)=datan(rcc(2)/rcc(1))
 else if(rcc(1).lt.-eps) then
  rs(3)=datan(rcc(2)/rcc(1))+pi
 else
  rs(3)=pi/2.d0*rsign(rcc(2))
 endif


endif



end subroutine cart2spher

subroutine spher2cart(rs,rcc)
real*8, intent(out)   :: rcc(3)
real*8, intent(in)   :: rs(3)

rcc(1)=rs(1)*dsin(rs(2))*dcos(rs(3))
rcc(2)=rs(1)*dsin(rs(2))*dsin(rs(3))
rcc(3)=rs(1)*dcos(rs(2))


end subroutine spher2cart

!subroutine interp_uniform(val, k)
!real*8,  intent(inout)           :: val(:,:)
!real*8, allocatable              :: a(:,:)
!logical, intent(inout)           :: k(:,:)
!integer                          :: ir,jr,therow,nx,ny
!
!nx=size(x,1)
!ny=size(x,2)
!
!allocate(a(nx*ny,nx*ny))
!
!
!do ir=2,nx-1        !!!!!!!!!!!!!!! internal points
! do jr=2,ny-1
!   therow=(ir-1)*ny + jr
!
!   a(therow,therow)=1.d0
!
!   if(known(ir,jr))then
!    b(therow)=val(ir,jr)
!   else
!
!
!    if(known(ir-1,jr))then
!      b(therow)=b(therow)+0.25d0*val(ir-1,jr)
!    else
!      a(therow,therow-ny)=-0.25d0
!    endif
!
!
!    if(known(ir+1,jr))then
!      b(therow)=b(therow)+0.25d0*val(ir+1,jr)
!    else
!      a(therow,therow+ny)=-0.25d0
!    endif
!
!    if(known(ir,jr-1))then
!      b(therow)=b(therow)+0.25d0*val(ir,jr-1)
!    else
!      a(therow,therow-1)=-0.25d0
!    endif
!
!    if(known(ir,jr+1))then
!      b(therow)=b(therow)+0.25d0*val(ir,jr+1)
!    else
!      a(therow,therow+1)=-0.25d0
!    endif
!
!   endif
!
! enddo
!enddo
!
!do ir=2,nx-1        !!!!!!!!!!!!!!! lateral columns
!do jr=1,ny,ny-1
!  therow=(ir-1)*ny + jr
!  a(therow,therow)=1.d0
!
!   if(known(ir,jr))then
!    b(therow)=val(ir,jr)
!   else
!
!    if(known(ir-1,jr))then
!      b(therow)=b(therow)+0.5d0*val(ir-1,jr)
!    else
!      a(therow,therow-ny)=-0.5d0
!    endif
!
!    if(known(ir+1,jr))then
!      b(therow)=b(therow)+0.5d0*val(ir+1,jr)
!    else
!      a(therow,therow+ny)=-0.5d0
!    endif
!
!   endif
!
! enddo
!enddo
!
!do ir=1,nx,nx-1        !!!!!!!!!!!!!!! top and bottom rows
! do jr=2,ny-1
!  therow=(ir-1)*ny + jr
!  a(therow,therow)=1.d0
!
!   if(known(ir,jr))then
!    b(therow)=val(ir,jr)
!   else
!
!    if(known(ir,jr-1))then
!      b(therow)=b(therow)+0.5d0*val(ir,jr-1)
!    else
!      a(therow,therow-1)=-0.5d0
!    endif
!
!    if(known(ir,jr+1))then
!      b(therow)=b(therow)+0.5d0*val(ir,jr+1)
!    else
!      a(therow,therow+1)=-0.5d0
!    endif
!
!   endif
!
! enddo
!enddo
!
!
!ir=1    !!!!!!!!!!! upper left corner
!jr=1
!
!  therow=(ir-1)*ny + jr
!  a(therow,therow)=1.d0
!
!  if(known(ir,jr))then
!   b(therow)=val(ir,jr)
!  else
!
!   if(known(ir,jr+1))then
!     b(therow)=b(therow)+0.5d0*val(ir,jr+1)
!   else
!     a(therow,therow+1)=-0.5d0
!  endif
!
!  if(known(ir+1,jr))then
!    b(therow)=b(therow)+0.5d0*val(ir+1,jr)
!  else
!    a(therow,therow+ny)=-0.5d0
!  endif
!
! endif
!ir=1    !!!!!!!!!!! upper right corner
!jr=ny
!
! therow=(ir-1)*ny + jr
! a(therow,therow)=1.d0
!
! if(known(ir,jr))then
!  b(therow)=val(ir,jr)
! else
!
!  if(known(ir,jr-1))then
!    b(therow)=b(therow)+0.5d0*val(ir,jr-1)
!  else
!    a(therow,therow-1)=-0.5d0
!  endif
!
!  if(known(ir+1,jr))then
!    b(therow)=b(therow)+0.5d0*val(ir+1,jr)
!  else
!    a(therow,therow+ny)=-0.5d0
!  endif
!
! endif
!
!
!ir=nx    !!!!!!!!!!! lower left corner
!jr=1
!
! therow=(ir-1)*ny + jr
! a(therow,therow)=1.d0
!
! if(known(ir,jr))then
!  b(therow)=val(ir,jr)
! else
!
!  if(known(ir,jr+1))then
!    b(therow)=b(therow)+0.5d0*val(ir,jr+1)
!  else
!    a(therow,therow+1)=-0.5d0
!  endif
!
!   if(known(ir-1,jr))then
!     b(therow)=b(therow)+0.5d0*val(ir-1,jr)
!   else
!     a(therow,therow-ny)=-0.5d0
!   endif
!
! endif
!
!
!ir=nx    !!!!!!!!!!! lower right corner
!jr=ny
!
! therow=(ir-1)*ny + jr
! a(therow,therow)=1.d0
!
! if(known(ir,jr))then
!  b(therow)=val(ir,jr)
! else
!
!  if(known(ir,jr-1))then
!    b(therow)=b(therow)+0.5d0*val(ir,jr-1)
!  else
!    a(therow,therow-1)=-0.5d0
!  endif
!
!   if(known(ir-1,jr))then
!     b(therow)=b(therow)+0.5d0*val(ir-1,jr)
!   else
!     a(therow,therow-ny)=-0.5d0
!   endif
!
! endif
!
!call dgetrf(nx*ny,nx*ny,a,nx*ny,ipiv,info)
!call dgetrs('N',nx*ny,1,a,nx*ny,ipiv,b,nx*ny,info)
!
!do ir=1, nx
! do jr=1,ny
!  x(ir,jr)=b((ir-1)*ny+jr)
! enddo
!enddo




!deallocate(known)
!deallocate(val)
!deallocate(b)
!deallocate(ipiv)
!deallocate(a)

!end subroutine interp_uniform

subroutine interp_scattered(points,x,f,dx)

integer, allocatable   :: ipiv(:)
real*8, allocatable    :: val(:,:),a(:,:),b(:),row(:)
real*8, intent(in)     :: f
real*8, intent(out)    :: dx
real*8, intent(inout)     :: points(:,:)
real*8,allocatable, intent(out)    :: x(:,:)
logical, allocatable   :: known(:,:)
integer                :: i,j,nx,ny,nz,vi(3),info,iunit,ir,jr,ic,jc,therow,np
integer                :: ldab,thej,kl,ku
real*8                 :: v1(3),v2(3)
real*8                 :: d1,dm,xmin,xmax,ymin,ymax,zmin,zmax,db(3,3),t(3)
real*8                 :: lx,ly

np=size(points,1)


dm=10.d0 
v1(1:2)=points(1,1:2)
v1(3)=0.d0
v2(3)=0.d0
do i=2,np
 v2(1:2)=points(i,1:2)
 d1=(norm(v1-v2))
 if(d1.lt.dm) dm=d1 
enddo

dx=dm*f

xmin=minval(points(1:np,1)) 
xmax=maxval(points(1:np,1)) 
ymin=minval(points(1:np,2)) 
ymax=maxval(points(1:np,2)) 
zmin=minval(points(1:np,3))
zmax=maxval(points(1:np,3))

lx=xmax-xmin
ly=ymax-ymin

nx=ceiling(lx/dx) + 1
ny=ceiling(ly/dx) + 1

nz=1
allocate(known(nx,ny))
allocate(val(nx,ny))
allocate(b(nx*ny))
allocate(ipiv(nx*ny))
allocate(x(nx,ny))
b=0.d0
val=0.d0
known=.false.

db=0
db(1,1)=dx
db(2,2)=dx
db(3,3)=1.d0

t(1)= 0.d0*dx -xmin
t(2)= 0.d0*dx -ymin
t(3)=0.d0

do i=1,np
 points(i,1:3)=points(i,1:3) + t  ! set origin
enddo


v1=0.d0
do i=1,np    ! identify known grid points
 v1(1)=points(i,1)
 v1(2)=points(i,2)
 call ijk(vi,v1,db)
 known(vi(1),vi(2))=.true.
 val(vi(1),vi(2))=points(i,3)
enddo

if(nx*ny.le.6400) then

allocate(a(nx*ny,nx*ny))
a=0.d0

do i=1,nx
 do j=1,ny

 therow=(i-1)*ny+j
 call interp_def_row(known,a(therow,1:nx*ny),nx,ny,i,j)
 call interp_def_b(val,known,b(therow),nx,ny,i,j)

 enddo
enddo




call dgetrf(nx*ny,nx*ny,a,nx*ny,ipiv,info)
call dgetrs('N',nx*ny,1,a,nx*ny,ipiv,b,nx*ny,info)

deallocate(a)

else
kl=ny
ku=ny
ldab=2*kl+ku+1
allocate(a(ldab,nx*ny))
allocate(row(nx*ny))
a=0.d0

do i=1,nx
 do j=1,ny
  therow=(i-1)*ny+j
  call interp_def_b(val,known,b(therow),nx,ny,i,j)
  call interp_def_row(known,row,nx,ny,i,j)
  do thej=max(1,therow-ku),min(nx*ny,therow+kl)
   a(kl+ku+1+therow-thej,thej)=row(thej) 
  enddo
 
 enddo
enddo

deallocate(row)

call dgbtrf(nx*ny,nx*ny,ny,ny,a,ldab,ipiv,info)
call dgbtrs("N",nx*ny,kl,ku,1,a,ldab,ipiv,b,nx*ny,info)

endif

do ir=1, nx       
 do jr=1,ny
  x(ir,jr)=b((ir-1)*ny+jr)
 enddo
enddo

deallocate(known)
deallocate(val)
deallocate(b)
deallocate(ipiv)

end subroutine interp_scattered

SUBROUTINE spline(x,y,n,yp1,ypn,y2)
 implicit none
 INTEGER,intent(in)            :: n
 double precision, intent(in)  :: yp1,ypn,x(n),y(n)
 double precision, intent(out) :: y2(n)
! Given arrays x(1:n) and y(1:n) containing a tabulated function,
! i.e., yi = f(xi), with x1 < x2 < ::: < xN, and given values yp1
! and ypn for the first derivative of the interpolating function at
! points 1 and n, respectively, this routine returns an array 
! y2(1:n) of length n which contains the second derivatives of the
! interpolating function at the tabulated points xi. If yp1 and/or
! ypn are equal to 1 x 1030 or larger, the routine is signaled to
! set the corresponding boundary condition for a natural spline,
! with zero second derivative on that boundary.
! Parameter: NMAX is the largest anticipated value of n.
 INTEGER NMAX
 PARAMETER (NMAX=1500)
 INTEGER i,k
 double precision p,qn,sig,un,u(NMAX)
 !print *, 'start spline'
 if (yp1.gt..99e30) then ! The lower boundary condition is set 
                        ! either to be \natural"
  y2(1)=0. 
  u(1)=0.
 else ! or else to have a specified first derivative.
  y2(1)=-0.5
  u(1)=(3./(x(2)-x(1)))*((y(2)-y(1))/(x(2)-x(1))-yp1)
 endif
 i=2
 do while (i<=n-1) !This is the decomposition loop of the tridiagonal
              ! algorithm. y2 and u are used for
              ! temporary storage of the decomposed factors.
  sig=(x(i)-x(i-1))/(x(i+1)-x(i-1))
  p=sig*y2(i-1)+2.
  y2(i)=(sig-1.)/p
  u(i)=(6.*((y(i+1)-y(i))/(x(i+1)-x(i))-(y(i)-y(i-1))&
   &/(x(i)-x(i-1)))/(x(i+1)-x(i-1))-sig*u(i-1))/p
  ! print *, 'mid spline, u(',i,') ',u(i),' sig ',sig,'y2(i): ',y2(i),'x(i) ',x(i),'x(i+1) ',x(i+1),'x(i-1) ',x(i-1)
  i=i+1  
 end do 
 
 if (ypn.gt..99e30) then ! The upper boundary condition is set
                        ! either  to be \natural"
  qn=0. 
  un=0.
 else ! or else to have a specified first derivative.
  qn=0.5
  un=(3./(x(n)-x(n-1)))*(ypn-(y(n)-y(n-1))/(x(n)-x(n-1)))
 endif
 y2(n)=(un-qn*u(n-1))/(qn*y2(n-1)+1.)
 k=n-1
 do while (k>=1) ! This is the backsubstitution loop of the
                 ! tridiagonal algorithm.
 y2(k)=y2(k)*y2(k+1)+u(k) 
 k=k-1
! print *, 'end spline, y2(',k,'): ',y2(k)
end do 
return
END SUBROUTINE spline
  

SUBROUTINE splint(xa,ya,y2a,n,x,y)
 implicit none
 
 INTEGER,intent(in) :: n
 double precision,intent(in)  :: x,xa(n),y2a(n),ya(n)
 double precision,intent(out) :: y
 ! Given the arrays xa(1:n) and ya(1:n) of length n, 
 ! which tabulate a function (with the xai 's in order), and given
 ! the array y2a(1:n), which is the output from spline above,
 ! and given a value of x, this routine returns a cubic-spline
 ! interpolated value y.
 INTEGER k,khi,klo
 double precision a,b,h
 klo=1 ! We will find the right place in the table by means of
 ! bisection. This is optimal if sequential calls to this routine
 ! are at random values of x. If sequential calls are in order, and
 ! closely spaced, one would do better to store previous values of
 ! klo and khi and test if they remain appropriate on the next call.
khi=n
1 if (khi-klo.gt.1) then
   k=(khi+klo)/2
   if(xa(k).gt.x) then
    khi=k
   else
    klo=k
   endif
   goto 1
  endif ! klo and khi now bracket the input value of x.
 h=xa(khi)-xa(klo)
 if (h.eq.0.) pause 'bad xa input in splint' ! The xa's must be  distinct.
 a=(xa(khi)-x)/h ! Cubic spline polynomial is now evaluated.
 b=(x-xa(klo))/h
 y=a*ya(klo)+b*ya(khi)+&
   &((a**3-a)*y2a(klo)+(b**3-b)*y2a(khi))*(h**2)/6.
 return
END subroutine splint

end module operations
